@@ -95,13 +95,13 @@ jobs:
     - {toolchain: gcc-8, target_arch: i386, kconfig: tinyconfig}
     - {toolchain: gcc-9, target_arch: i386, kconfig: tinyconfig}
     - {toolchain: gcc-10, target_arch: i386, kconfig: tinyconfig}
-  test: {device: qemu-i386, tests: [ltp-smoke]}
+  test: {device: qemu-i386, tests: [ltp-smoke], overlay:[[overlay.tgz,/tmp],[overlay2.tgz]]}
 
 - builds:
     - {toolchain: clang-10, target_arch: i386, kconfig: tinyconfig}
     - {toolchain: clang-11, target_arch: i386, kconfig: tinyconfig}
     - {toolchain: clang-nightly, target_arch: i386, kconfig: tinyconfig}
-  test: {device: qemu-i386}
+  test: {device: qemu-i386, overlay: overlay.tgz}
 
 - build: {toolchain: clang-nightly, target_arch: i386, kconfig: tinyconfig}
   tests:
@@ -140,6 +140,79 @@ def plan_config(tmp_path, sample_plan_config):
     return c
 
 
+@pytest.fixture
+def sample_plan_with_sanity():
+    return """
+version: 1
+name: Simple sanity plan
+description: A simple sanity plan
+jobs:
+- build: {toolchain: clang-nightly, target_arch: i386, kconfig: tinyconfig}
+  sanity_test: {device: qemu-i386}
+  tests:
+    - {device: qemu-i386}
+    - {device: qemu-i386, tests: [ltp-smoke]}
+
+- builds:
+    - {toolchain: gcc-8, target_arch: i386, kconfig: tinyconfig}
+    - {toolchain: gcc-9, target_arch: i386, kconfig: tinyconfig}
+    - {toolchain: gcc-10, target_arch: i386, kconfig: tinyconfig}
+  sanity_test: {device: qemu-i386, parameters: {HELLO: WORLD}}
+  tests:
+    - {device: qemu-i386, parameters: {HELLO: WORLD}}
+    - {device: qemu-i386, tests: [ltp-syscalls], sharding: 3}
+    - {device: qemu-i386, tests: [ltp-commands], sharding: 1, parameters: {HELLO: WORLD}}
+- build: {toolchain: clang-nightly, target_arch: i386, kconfig: tinyconfig}
+  tests:
+    - {device: qemu-i386}
+    - {device: qemu-i386, tests: [ltp-smoke]}
+
+- tests:
+    - {kernel: https://storage.tuxboot.com/arm64/Image, device: qemu-arm64, tests: [ltp-smoke]}
+    - {kernel: https://storage.tuxboot.com/i386/bzImage, device: qemu-i386, tests: [ltp-smoke]}
+    - {kernel: https://storage.tuxboot.com/mips64/vmlinux, device: qemu-mips64, tests: [ltp-smoke]}
+    - {kernel: https://storage.tuxboot.com/ppc64/vmlinux, device: qemu-ppc64, tests: [ltp-smoke]}
+    - kernel: https://storage.tuxboot.com/riscv64/Image
+      device: qemu-riscv64
+      bios: https://storage.tuxboot.com/riscv64/fw_jump.elf
+      tests: [ltp-smoke]
+    - kernel: https://storage.tuxboot.com/x86_64/bzImage
+      device: qemu-x86_64
+      rootfs: https://storage.tuxboot.com/x86_64/rootfs.ext4.zst
+      tests: [ltp-smoke]
+  sanity_test: {kernel: https://storage.tuxboot.com/arm64/Image, device: qemu-arm64, tests: [ltp-smoke]}
+- tests:
+    - {kernel: https://storage.tuxboot.com/arm64/Image, device: qemu-arm64, tests: [ltp-smoke]}
+    - {kernel: https://storage.tuxboot.com/i386/bzImage, device: qemu-i386, tests: [ltp-smoke]}
+"""
+
+
+@pytest.fixture
+def sanity_plan_config(tmp_path, sample_plan_with_sanity):
+    c = tmp_path / "sanity_plan.yml"
+    c.write_text(sample_plan_with_sanity)
+    return c
+
+
+@pytest.fixture
+def sample_tuxtrigger_config():
+    return """
+repositories:
+- url: https://some/test/repo/url
+  squad_group: ~first_name.lastname
+  branches:
+  - name: test_branch
+    plan: test_plan.yaml
+"""
+
+
+@pytest.fixture
+def tuxtrigger_config(tmp_path, sample_tuxtrigger_config):
+    c = tmp_path / "config.yml"
+    c.write_text(sample_tuxtrigger_config)
+    return c
+
+
 @pytest.fixture
 def sample_bake_plan_config():
     return """
@@ -226,6 +299,46 @@ def bake_plan_extended_config(tmp_path, sample_bake_plan_extended_config):
     return bake_config
 
 
+@pytest.fixture
+def sample_bake_plan_config_with_test():
+    return """
+common: &commondata
+  "container": "ubuntu-20.04"
+  "distro": "rpb"
+  "envsetup": "setup-environment"
+  "machine": "dragonboard-845c"
+  "sources": {
+    "repo": {
+      "branch": "qcom/dunfell",
+      "manifest": "default.xml",
+      "url": "https://github.com/96boards/oe-rpb-manifest.git",
+    }
+  }
+  "target": "rpb-console-image rpb-console-image-test rpb-desktop-image rpb-desktop-image-test"
+version: 1
+name: armv7 validation
+description: Build and test linux kernel for armv7
+jobs:
+- name: armv7
+  bake: { <<: *commondata, "machine": "ledge-multi-armv7"}
+
+- name: lt-qcom
+  bakes:
+    - { <<: *commondata}
+    - { <<: *commondata}
+  tests:
+    - {device: qemu-i386}
+    - {device: qemu-i386, tests: [ltp-smoke]}
+"""
+
+
+@pytest.fixture
+def bake_plan_config_with_test(tmp_path, sample_bake_plan_config_with_test):
+    bake_config = tmp_path / "sample_bake_plan.yml"
+    bake_config.write_text(sample_bake_plan_config_with_test)
+    return bake_config
+
+
 @pytest.fixture
 def sample_plan_unknown_version():
     return """
@@ -337,3 +450,47 @@ def build(build_attrs):
     b = tuxsuite.build.Build(**build_attrs)
     b.uid = "myuid"
     return b
+
+
+@pytest.fixture
+def single_build_plan():
+    return """
+version: 1
+name: Simple sanity plan
+description: A simple sanity plan
+jobs:
+- build: {toolchain: clang-nightly, target_arch: i386, kconfig: tinyconfig}
+"""
+
+
+@pytest.fixture
+def build_plan(tmp_path, single_build_plan):
+    m = tmp_path / "build-plan.yaml"
+    m.write_text(single_build_plan)
+    return str(m)
+
+
+@pytest.fixture
+def single_test_plan():
+    return """
+version: 1
+name: Simple sanity plan
+description: A simple sanity plan
+jobs:
+- test:
+    device: qemu-i386
+    tests: [ltp-smoke]
+    dtb: https://storage.tuxsuite.com/grp/prj/tests/uid/dtbs/am57xx-beagle-x15.dtb
+    parameters:
+      ITERATIONS: 2
+      SKIPFILE: skipfile-lkft.yaml
+      KSELFTEST: https://storage.tuxsuite.com/grp/prj/tests/uid/kselftest.tar.xz
+      PERF: https://storage.tuxsuite.com/grp/prj/tests/uid/perf.tar.xz
+"""
+
+
+@pytest.fixture
+def test_plan(tmp_path, single_test_plan):
+    m = tmp_path / "test-plan.yaml"
+    m.write_text(single_test_plan)
+    return str(m)
