@@ -213,10 +213,13 @@ class TestBitbake:
             "no_cache": False,
             "is_public": True,
             "callback": None,
+            "callback_headers": None,
             "bblayers_conf": [],
             "container": bitbake.build_definition.container,
             "manifest_file": None,
             "pinned_manifest": None,
+            "kas_override": None,
+            "notify_emails": [],
         }
         assert bitbake.generate_build_request() == (expected_data, {})
 
@@ -248,6 +251,7 @@ class TestBitbake:
                 "no_cache": False,
                 "is_public": True,
                 "callback": None,
+                "callback_headers": None,
                 "artifacts": [],
                 "environment": {},
                 "extraconfigs": [],
@@ -257,32 +261,64 @@ class TestBitbake:
                 "manifests": {},
                 "pinned_manifest": None,
                 "manifest_file": None,
+                "kas_override": None,
+                "notify_emails": [],
             },
         )
 
-    def test_handle_manifest(self, mocker, sample_manifest, sample_manifest_file):
-        guess = mocker.patch(
-            "mimetypes.guess_type", return_value=("application/xml", None)
-        )
-        result = tuxsuite.build.handle_manifest(sample_manifest_file)
-        assert guess.call_count == 1
+    def test_handle_manifest(
+        self, capsys, sample_manifest, sample_manifest_file, tmp_path
+    ):
+        result = tuxsuite.build.handle_attachment(sample_manifest_file)
         assert isinstance(result, type(()))
         assert isinstance(type(result[0]), type(str))
         assert base64.b64decode(result[1]) == sample_manifest.encode("utf-8")
 
-    def test_handle_manifest_unknown(self, mocker, sample_manifest_file):
-        guess = mocker.patch(
-            "mimetypes.guess_type", return_value=("application/unknown", None)
-        )
-        with pytest.raises(Exception) as e:
-            tuxsuite.build.handle_manifest(sample_manifest_file)
+        # if manifest/ kas override file is not a file
+        file_path = "/some/dummy/folder/"
+        with pytest.raises(SystemExit):
+            tuxsuite.build.handle_attachment(file_path)
+        output, error = capsys.readouterr()
+        assert "Is provided file a valid xml/yaml file?" in error
 
-        assert guess.call_count == 1
-        assert str(e.value) == "Manifest format unknown"
+        # if kas override is passed but extension is not .yaml/.yml
+        with pytest.raises(SystemExit):
+            tuxsuite.build.handle_attachment(sample_manifest_file, file_type="yaml")
+        output, error = capsys.readouterr()
+        assert "does not have yaml file extension" in error
+
+        # if manifest is passed but extension is not .xml
+        unknown_manifest = tmp_path / "manifest.yml"
+        unknown_manifest.write_text("<xml></xml>")
+        with pytest.raises(SystemExit):
+            tuxsuite.build.handle_attachment(str(unknown_manifest))
+        output, error = capsys.readouterr()
+        assert "does not have xml file extension" in error
+
+        # if invalid manifest is passed
+        invalid_manifest = tmp_path / "invalid_manifest.xml"
+        invalid_manifest.write_text("<xml><//xml>")
+        with pytest.raises(SystemExit):
+            tuxsuite.build.handle_attachment(str(invalid_manifest))
+        output, error = capsys.readouterr()
+        assert "Error: Error parsing" in error
+
+        # if invalid kas override file is passed
+        invalid_override_file = tmp_path / "invalid_manifest.yaml"
+        invalid_override_file.write_text(
+            """
+        work: 555-5678
+        - mobile: 555-9012
+        %invalid-key: some value"""
+        )
+        with pytest.raises(SystemExit):
+            tuxsuite.build.handle_attachment(str(invalid_override_file), True)
+        output, error = capsys.readouterr()
+        assert "Error: Error parsing" in error
 
     def test_handle_manifest_url(self):
         url = "https://test.example.com/url"
-        result = tuxsuite.build.handle_manifest(url)
+        result = tuxsuite.build.handle_attachment(url)
         assert isinstance(result, type(()))
         assert isinstance(type(result[0]), type(str))
         assert result[0] == url
@@ -420,7 +456,7 @@ class TestBakeWait:
 
 class TestBuild:
     def test_kconfig(self, build):
-        assert type(build.kconfig) == list
+        assert isinstance(build.kconfig, list)
 
     @pytest.mark.parametrize(
         "attr,value",
@@ -433,20 +469,22 @@ class TestBuild:
             ("toolchain", None),
         ),
     )
-    def test_requires_mandatory_attributes(self, build_attrs, attr, value):
+    def test_requires_mandatory_attributes(self, build_attrs, attr, value, capsys):
         build_attrs[attr] = value
-        with pytest.raises(AssertionError) as assertion:
+        with pytest.raises(SystemExit):
             tuxsuite.build.Build(**build_attrs)
+        output, error = capsys.readouterr()
         if attr == "target_arch":
-            assert "target-arch" in str(assertion)
+            assert "target-arch" in error
         else:
-            assert attr in str(assertion)
+            assert attr in error
 
-    def test_validates_git_url(self, build_attrs):
+    def test_validates_git_url(self, build_attrs, capsys):
         build_attrs["git_repo"] = "ssh://foo.com:bar.git"
-        with pytest.raises(AssertionError) as assertion:
+        with pytest.raises(SystemExit):
             tuxsuite.build.Build(**build_attrs)
-        assert "git url must be in the form" in str(assertion)
+        output, error = capsys.readouterr()
+        assert "git url must be in the form" in error
 
     def test_headers(self, build):
         assert build.headers["Content-Type"] == "application/json"
@@ -461,12 +499,13 @@ class TestBuild:
         build = tuxsuite.build.Build(**build_attrs)
         assert build.git_sha == "deadbeef"
 
-    def test_git_ref_or_git_sha_required(self, build_attrs):
+    def test_git_ref_or_git_sha_required(self, build_attrs, capsys):
         del build_attrs["git_ref"]
-        with pytest.raises(AssertionError) as assertion:
+        with pytest.raises(SystemExit):
             tuxsuite.build.Build(**build_attrs)
-        assert "git_ref" in str(assertion)
-        assert "git_sha" in str(assertion)
+        output, error = capsys.readouterr()
+        assert "git_ref" in error
+        assert "git_sha" in error
 
     def test_build_name(self, build_attrs):
         del build_attrs["build_name"]
@@ -842,18 +881,6 @@ class TestBuildWatch(TestWatch):
         states = self.watch(build)
         assert [s.state for s in states] == ["queued", "complete"]
 
-    def test_timeout(self, build, mocker, get_request):
-        get_request.side_effect = iter(
-            lambda: {
-                "state": "queued",
-                "result": "unknown",
-                "tuxbuild_status": "queued",
-            },
-            1,
-        )
-        with pytest.raises(tuxsuite.exceptions.Timeout):
-            self.watch(build)
-
     def test_resists_unknown_state(self, build, build_statuses):
         build_statuses.insert(
             2,
@@ -918,6 +945,8 @@ def results(build_attrs):
         tuxapi_url=build_attrs["tuxapi_url"],
         token=build_attrs["token"],
         uid="1sewrBhxNVbsURAKBjeXX8pyjwY",
+        lava_test_plans_project=None,
+        lab=None,
     )
 
 
@@ -990,9 +1019,8 @@ class TestResults:
 
 class TestHandlePatch:
     def test_handle_patch_unknown(self):
-        with pytest.raises(Exception) as excinfo:
+        with pytest.raises(Exception):
             tuxsuite.build.handle_patch("/tmp/patch")
-        assert "No such file or directory" in str(excinfo)
 
     def test_handle_patch_mbox(self, mocker, sample_patch, sample_patch_file):
         guess = mocker.patch(
@@ -1072,6 +1100,9 @@ def test_attrs():
         "token": "test_token",
         "kbapi_url": "http://test/foo",
         "tuxapi_url": "http://tuxapi",
+        "test_name": None,
+        "lava_test_plans_project": None,
+        "lab": None,
     }
 
 
@@ -1084,7 +1115,7 @@ def test(test_attrs):
 
 class TestTest:
     def test_test_str(self, test):
-        assert type(test.tests) == list
+        assert isinstance(test.tests, list)
         print(test)
 
     def test_submit_test(self, test, test_attrs, mocker):
@@ -1111,5 +1142,9 @@ class TestTest:
                 "scp_romfw": None,
                 "fip": None,
                 "tests": test_attrs["tests"],
+                "lava_test_plans_project": None,
+                "lab": None,
+                "host": "x86_64",
+                "is_public": True,
             },
         )
