@@ -4,6 +4,7 @@ import os
 import json
 import pytest
 import sys
+import yaml
 import tuxsuite.cli
 import tuxsuite.build
 
@@ -31,21 +32,15 @@ def tuxsuite_config(tmp_path, monkeypatch, tuxauth):
 @pytest.fixture
 def sample_bitbake_template():
     return """
-{
-  "container": "ubuntu-20.04",
-  "envsetup": "poky/oe-init-build-env",
-  "distro": "poky",
-  "machine": "qemux86-64",
-  "target": "core-image-minimal",
-  "sources": {
-    "git_trees": [
-      {
-        "url": "git://git.yoctoproject.org/poky",
-        "branch": "dunfell"
-      }
-    ]
-  }
-}
+container: ubuntu-20.04
+envsetup: poky/oe-init-build-env
+distro: poky
+machine: qemux86-64
+target: core-image-minimal
+sources:
+  git_trees:
+    - url: git://git.yoctoproject.org/poky
+      branch: dunfell
 """
 
 
@@ -190,7 +185,7 @@ class TestBakeCli:
     def test_bake(
         self, mocker, monkeypatch, tuxsuite_config, tmp_path, sample_bitbake_template
     ):
-        template = tmp_path / "yocto.json"
+        template = tmp_path / "yocto.yaml"
         template.write_text(sample_bitbake_template)
         build = mocker.patch("tuxsuite.Bitbake")
         monkeypatch.setattr(sys, "argv", ["tuxsuite", "bake", "submit", f"{template}"])
