@@ -216,13 +216,7 @@ def test_build_handle_config(mocker, config, response, monkeypatch):
     assert config_req.call_count == 1
 
 
-def test_build_handle_get(
-    mocker,
-    build_json,
-    config,
-    response,
-    monkeypatch,
-):
+def test_build_handle_get(mocker, build_json, config, response, monkeypatch, tmp_path):
     monkeypatch.setattr(
         sys, "argv", ["tuxsuite", "build", "get", "205Yag6znViIi4oLWFVDxJC4avb"]
     )
@@ -232,7 +226,7 @@ def test_build_handle_get(
     tuxsuite.cli.main()
     assert get_req.call_count == 1
 
-    # Test json out
+    # Test --json
     monkeypatch.setattr(
         sys,
         "argv",
@@ -251,6 +245,131 @@ def test_build_handle_get(
         tuxsuite.cli.main()
     assert get_req.call_count == 1
 
+    # Test --json-out
+    json_path = f"{tmp_path}/build.json"
+    monkeypatch.setattr(
+        sys,
+        "argv",
+        [
+            "tuxsuite",
+            "build",
+            "get",
+            "205Yag6znViIi4oLWFVDxJC4avb",
+            "--json-out",
+            json_path,
+        ],
+    )
+    response.status_code = 200
+    response._content = build_json
+    get_req = mocker.patch("requests.get", return_value=response)
+    tuxsuite.cli.main()
+    assert get_req.call_count == 1
+    assert len(json.load(open(json_path, "r"))) > 0
+
+    # case: --list-artifacts
+    monkeypatch.setattr(
+        sys,
+        "argv",
+        [
+            "tuxsuite",
+            "build",
+            "get",
+            "205Yag6znViIi4oLWFVDxJC4avb",
+            "--list-artifacts",
+        ],
+    )
+    response.status_code = 200
+    response._content = build_json
+    get_req = mocker.patch("requests.get", return_value=response)
+    tuxsuite.cli.main()
+    assert get_req.call_count == 2
+    assert "builds.stylesen.dev.tuxbuild.com" in get_req.call_args[0][0]
+
+    # case: --list-artifacts with path
+    monkeypatch.setattr(
+        sys,
+        "argv",
+        [
+            "tuxsuite",
+            "build",
+            "get",
+            "205Yag6znViIi4oLWFVDxJC4avb",
+            "--list-artifacts",
+            "dir_A",
+        ],
+    )
+    response.status_code = 200
+    response._content = build_json
+    get_req = mocker.patch("requests.get", return_value=response)
+    tuxsuite.cli.main()
+    assert get_req.call_count == 2
+    assert "builds.stylesen.dev.tuxbuild.com" in get_req.call_args[0][0]
+    assert "/dir_A" in get_req.call_args[0][0]
+
+    # case: --list-artifacts with download option
+    monkeypatch.setattr(
+        sys,
+        "argv",
+        [
+            "tuxsuite",
+            "build",
+            "get",
+            "205Yag6znViIi4oLWFVDxJC4avb",
+            "--list-artifacts",
+            "-d",
+            str(tmp_path),
+        ],
+    )
+    response.status_code = 200
+    response._content = build_json
+    get_req = mocker.patch("requests.get", return_value=response)
+    tuxsuite.cli.main()
+    assert get_req.call_count == 2
+    assert "builds.stylesen.dev.tuxbuild.com" in get_req.call_args[0][0]
+
+    # case: --list-artifacts with path and download option
+    monkeypatch.setattr(
+        sys,
+        "argv",
+        [
+            "tuxsuite",
+            "build",
+            "get",
+            "205Yag6znViIi4oLWFVDxJC4avb",
+            "--list-artifacts",
+            "dir_A",
+            "-d",
+            str(tmp_path),
+        ],
+    )
+    response.status_code = 200
+    response._content = build_json
+    get_req = mocker.patch("requests.get", return_value=response)
+    tuxsuite.cli.main()
+    assert get_req.call_count == 2
+    assert "builds.stylesen.dev.tuxbuild.com" in get_req.call_args[0][0]
+    assert "/dir_A" in get_req.call_args[0][0]
+
+    # case: --download-artifacts with path
+    monkeypatch.setattr(
+        sys,
+        "argv",
+        [
+            "tuxsuite",
+            "build",
+            "get",
+            "205Yag6znViIi4oLWFVDxJC4avb",
+            "--download-artifacts",
+            str(tmp_path),
+        ],
+    )
+    response.status_code = 200
+    response._content = build_json
+    get_req = mocker.patch("requests.get", return_value=response)
+    tuxsuite.cli.main()
+    assert get_req.call_count == 2
+    assert "builds.stylesen.dev.tuxbuild.com" in get_req.call_args[0][0]
+
 
 def test_build_handle_cancel_get(
     mocker, build_json_cancel, config, response, monkeypatch
@@ -304,18 +423,21 @@ def test_build_handle_cancel(mocker, build_json, config, response, monkeypatch):
     assert post_req.call_count == 1
 
 
-def test_build_handle_list(mocker, build_list_json, config, response, monkeypatch):
-    monkeypatch.setattr(sys, "argv", ["tuxsuite", "build", "list"])
+def test_build_handle_list(
+    mocker, build_list_json, config, response, monkeypatch, tmp_path
+):
+    monkeypatch.setattr(sys, "argv", ["tuxsuite", "build", "list", "--limit", "1"])
     monkeypatch.setattr("sys.stdout.isatty", lambda: True)
+    mocker.patch("builtins.input", side_effect=KeyboardInterrupt)
     response.status_code = 200
     response._content = build_list_json
     list_req = mocker.patch("requests.get", return_value=response)
     with pytest.raises(SystemExit) as exit:
         tuxsuite.cli.main()
     assert exit.value.code == 0
-    assert list_req.call_count == 2
+    assert list_req.call_count == 1
 
-    # Test json out
+    # Test --json
     monkeypatch.setattr(sys, "argv", ["tuxsuite", "build", "list", "--json"])
     response.status_code = 200
     response._content = build_list_json
@@ -331,6 +453,19 @@ def test_build_handle_list(mocker, build_list_json, config, response, monkeypatc
         tuxsuite.cli.main()
     assert list_req.call_count == 1
 
+    # Test --json-out
+    json_path = f"{tmp_path}/build.json"
+    monkeypatch.setattr(
+        sys, "argv", ["tuxsuite", "build", "list", "--json-out", json_path]
+    )
+    response.status_code = 200
+    response._content = build_list_json
+    list_req = mocker.patch("requests.get", return_value=response)
+    tuxsuite.cli.main()
+    assert exit.value.code == 0
+    assert list_req.call_count == 1
+    assert len(json.load(open(json_path, "r"))) > 0
+
 
 def test_build_handle_logs(mocker, config, response, monkeypatch):
     monkeypatch.setattr(
