@@ -205,13 +205,10 @@ def test_load_build_definition(get, response, capsys, tmp_path):
     content = {"sources": {"repo": {}}}
     response.status_code = 200
     response._content = json.dumps(content).encode("utf-8")
-    data = load_build_definition(definition_file)
-    assert data == content
-    out, _ = capsys.readouterr()
-    assert (
-        "⚠️ WARNING: Support for build definition files in JSON format is deprecated"
-        in out
-    )
+    with pytest.raises(SystemExit):
+        load_build_definition(definition_file)
+    _, err = capsys.readouterr()
+    assert "The build definition file is not valid. It must be in YAML format" in err
 
     # case: build definition URL: 400 response
     definition_file = (
@@ -226,10 +223,7 @@ def test_load_build_definition(get, response, capsys, tmp_path):
     definition_file = tmp_path / "build-definition.json"
     with open(definition_file, "w") as f:
         json.dump(content, f)
-    data = load_build_definition(definition_file)
-    assert data == content
-    out, _ = capsys.readouterr()
-    assert (
-        "⚠️ WARNING: Support for build definition files in JSON format is deprecated"
-        in out
-    )
+    with pytest.raises(SystemExit):
+        load_build_definition(definition_file)
+    _, err = capsys.readouterr()
+    assert "The build definition file is not valid. It must be in YAML format" in err
