@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import pytest
+from unittest.mock import ANY
 
 from tuxsuite.config import PlanConfig
 import tuxsuite.build
@@ -12,10 +13,12 @@ def test_plan_config(plan_config):
     cfg = PlanConfig("", "", plan_config)
     assert cfg.name == "Simple plan"
     assert cfg.description == "A simple plan"
+    assert cfg.plan_file == open(plan_config).read()
 
     cfg = PlanConfig("hello", "world", plan_config)
     assert cfg.name == "hello"
     assert cfg.description == "world"
+    assert cfg.plan_file == open(plan_config).read()
 
     assert len(cfg.plan) == 12
     assert cfg.plan[0]["build"] == {
@@ -36,42 +39,60 @@ def test_plan_config(plan_config):
         "target_arch": "i386",
         "toolchain": "gcc-8",
     }
-    assert cfg.plan[1]["tests"] == [{"device": "qemu-i386", "tests": ["ltp-smoke"]}]
+    assert cfg.plan[1]["tests"] == [
+        {
+            "device": "qemu-i386",
+            "tests": ["ltp-smoke"],
+            "overlay": [["overlay.tgz", "/tmp"], ["overlay2.tgz"]],
+        }
+    ]
 
     assert cfg.plan[2]["build"] == {
         "kconfig": "tinyconfig",
         "target_arch": "i386",
         "toolchain": "gcc-9",
     }
-    assert cfg.plan[2]["tests"] == [{"device": "qemu-i386", "tests": ["ltp-smoke"]}]
+    assert cfg.plan[2]["tests"] == [
+        {
+            "device": "qemu-i386",
+            "tests": ["ltp-smoke"],
+            "overlay": [["overlay.tgz", "/tmp"], ["overlay2.tgz"]],
+        }
+    ]
 
     assert cfg.plan[3]["build"] == {
         "kconfig": "tinyconfig",
         "target_arch": "i386",
         "toolchain": "gcc-10",
     }
-    assert cfg.plan[3]["tests"] == [{"device": "qemu-i386", "tests": ["ltp-smoke"]}]
+    assert cfg.plan[3]["tests"] == [
+        {
+            "device": "qemu-i386",
+            "tests": ["ltp-smoke"],
+            "overlay": [["overlay.tgz", "/tmp"], ["overlay2.tgz"]],
+        }
+    ]
 
     assert cfg.plan[4]["build"] == {
         "kconfig": "tinyconfig",
         "target_arch": "i386",
         "toolchain": "clang-10",
     }
-    assert cfg.plan[4]["tests"] == [{"device": "qemu-i386"}]
+    assert cfg.plan[4]["tests"] == [{"device": "qemu-i386", "overlay": "overlay.tgz"}]
 
     assert cfg.plan[5]["build"] == {
         "kconfig": "tinyconfig",
         "target_arch": "i386",
         "toolchain": "clang-11",
     }
-    assert cfg.plan[5]["tests"] == [{"device": "qemu-i386"}]
+    assert cfg.plan[5]["tests"] == [{"device": "qemu-i386", "overlay": "overlay.tgz"}]
 
     assert cfg.plan[6]["build"] == {
         "kconfig": "tinyconfig",
         "target_arch": "i386",
         "toolchain": "clang-nightly",
     }
-    assert cfg.plan[6]["tests"] == [{"device": "qemu-i386"}]
+    assert cfg.plan[6]["tests"] == [{"device": "qemu-i386", "overlay": "overlay.tgz"}]
 
     assert cfg.plan[7]["build"] == {
         "kconfig": "tinyconfig",
@@ -207,14 +228,203 @@ def test_plan_config(plan_config):
     ]
 
 
+def test_sanity_plan_config(sanity_plan_config):
+    cfg = PlanConfig("", "", sanity_plan_config)
+    assert cfg.name == "Simple sanity plan"
+    assert cfg.description == "A simple sanity plan"
+    assert cfg.plan_file == open(sanity_plan_config).read()
+
+    cfg = PlanConfig("hello", "world", sanity_plan_config)
+    assert cfg.name == "hello"
+    assert cfg.description == "world"
+    assert cfg.plan_file == open(sanity_plan_config).read()
+
+    assert len(cfg.plan) == 7
+    assert cfg.plan[0]["build"] == {
+        "kconfig": "tinyconfig",
+        "target_arch": "i386",
+        "toolchain": "clang-nightly",
+    }
+    assert cfg.plan[0]["sanity_test"] == {"device": "qemu-i386"}
+    assert cfg.plan[0]["tests"] == [
+        {"device": "qemu-i386"},
+        {"device": "qemu-i386", "tests": ["ltp-smoke"]},
+    ]
+
+    assert cfg.plan[1]["build"] == {
+        "kconfig": "tinyconfig",
+        "target_arch": "i386",
+        "toolchain": "gcc-8",
+    }
+    assert cfg.plan[1]["sanity_test"] == {
+        "device": "qemu-i386",
+        "parameters": {"HELLO": "WORLD"},
+    }
+    assert cfg.plan[1]["tests"] == [
+        {"device": "qemu-i386", "parameters": {"HELLO": "WORLD"}},
+        {
+            "device": "qemu-i386",
+            "tests": ["ltp-syscalls"],
+            "parameters": {"SHARD_NUMBER": 3, "SHARD_INDEX": 1},
+        },
+        {
+            "device": "qemu-i386",
+            "tests": ["ltp-syscalls"],
+            "parameters": {"SHARD_NUMBER": 3, "SHARD_INDEX": 2},
+        },
+        {
+            "device": "qemu-i386",
+            "tests": ["ltp-syscalls"],
+            "parameters": {"SHARD_NUMBER": 3, "SHARD_INDEX": 3},
+        },
+        {
+            "device": "qemu-i386",
+            "tests": ["ltp-commands"],
+            "parameters": {"HELLO": "WORLD", "SHARD_NUMBER": 1, "SHARD_INDEX": 1},
+        },
+    ]
+    assert cfg.plan[2]["build"] == {
+        "kconfig": "tinyconfig",
+        "target_arch": "i386",
+        "toolchain": "gcc-9",
+    }
+    assert cfg.plan[2]["sanity_test"] == {
+        "device": "qemu-i386",
+        "parameters": {"HELLO": "WORLD"},
+    }
+    assert cfg.plan[2]["tests"] == [
+        {"device": "qemu-i386", "parameters": {"HELLO": "WORLD"}},
+        {
+            "device": "qemu-i386",
+            "tests": ["ltp-syscalls"],
+            "parameters": {"SHARD_NUMBER": 3, "SHARD_INDEX": 1},
+        },
+        {
+            "device": "qemu-i386",
+            "tests": ["ltp-syscalls"],
+            "parameters": {"SHARD_NUMBER": 3, "SHARD_INDEX": 2},
+        },
+        {
+            "device": "qemu-i386",
+            "tests": ["ltp-syscalls"],
+            "parameters": {"SHARD_NUMBER": 3, "SHARD_INDEX": 3},
+        },
+        {
+            "device": "qemu-i386",
+            "tests": ["ltp-commands"],
+            "parameters": {"HELLO": "WORLD", "SHARD_NUMBER": 1, "SHARD_INDEX": 1},
+        },
+    ]
+    assert cfg.plan[3]["build"] == {
+        "kconfig": "tinyconfig",
+        "target_arch": "i386",
+        "toolchain": "gcc-10",
+    }
+    assert cfg.plan[3]["sanity_test"] == {
+        "device": "qemu-i386",
+        "parameters": {"HELLO": "WORLD"},
+    }
+    assert cfg.plan[3]["tests"] == [
+        {"device": "qemu-i386", "parameters": {"HELLO": "WORLD"}},
+        {
+            "device": "qemu-i386",
+            "tests": ["ltp-syscalls"],
+            "parameters": {"SHARD_NUMBER": 3, "SHARD_INDEX": 1},
+        },
+        {
+            "device": "qemu-i386",
+            "tests": ["ltp-syscalls"],
+            "parameters": {"SHARD_NUMBER": 3, "SHARD_INDEX": 2},
+        },
+        {
+            "device": "qemu-i386",
+            "tests": ["ltp-syscalls"],
+            "parameters": {"SHARD_NUMBER": 3, "SHARD_INDEX": 3},
+        },
+        {
+            "device": "qemu-i386",
+            "tests": ["ltp-commands"],
+            "parameters": {"HELLO": "WORLD", "SHARD_NUMBER": 1, "SHARD_INDEX": 1},
+        },
+    ]
+    assert cfg.plan[4]["build"] == {
+        "toolchain": "clang-nightly",
+        "target_arch": "i386",
+        "kconfig": "tinyconfig",
+    }
+    assert cfg.plan[4]["sanity_test"] == {}
+    assert cfg.plan[4]["tests"] == [
+        {"device": "qemu-i386"},
+        {"device": "qemu-i386", "tests": ["ltp-smoke"]},
+    ]
+
+    assert cfg.plan[5]["build"] is None
+    assert cfg.plan[5]["sanity_test"] == {
+        "kernel": "https://storage.tuxboot.com/arm64/Image",
+        "device": "qemu-arm64",
+        "tests": ["ltp-smoke"],
+    }
+
+    assert cfg.plan[5]["tests"] == [
+        {
+            "kernel": "https://storage.tuxboot.com/arm64/Image",
+            "device": "qemu-arm64",
+            "tests": ["ltp-smoke"],
+        },
+        {
+            "kernel": "https://storage.tuxboot.com/i386/bzImage",
+            "device": "qemu-i386",
+            "tests": ["ltp-smoke"],
+        },
+        {
+            "kernel": "https://storage.tuxboot.com/mips64/vmlinux",
+            "device": "qemu-mips64",
+            "tests": ["ltp-smoke"],
+        },
+        {
+            "kernel": "https://storage.tuxboot.com/ppc64/vmlinux",
+            "device": "qemu-ppc64",
+            "tests": ["ltp-smoke"],
+        },
+        {
+            "kernel": "https://storage.tuxboot.com/riscv64/Image",
+            "bios": "https://storage.tuxboot.com/riscv64/fw_jump.elf",
+            "device": "qemu-riscv64",
+            "tests": ["ltp-smoke"],
+        },
+        {
+            "kernel": "https://storage.tuxboot.com/x86_64/bzImage",
+            "device": "qemu-x86_64",
+            "rootfs": "https://storage.tuxboot.com/x86_64/rootfs.ext4.zst",
+            "tests": ["ltp-smoke"],
+        },
+    ]
+    assert cfg.plan[6]["build"] is None
+    assert cfg.plan[6]["sanity_test"] == {}
+    assert cfg.plan[6]["tests"] == [
+        {
+            "kernel": "https://storage.tuxboot.com/arm64/Image",
+            "device": "qemu-arm64",
+            "tests": ["ltp-smoke"],
+        },
+        {
+            "kernel": "https://storage.tuxboot.com/i386/bzImage",
+            "device": "qemu-i386",
+            "tests": ["ltp-smoke"],
+        },
+    ]
+
+
 def test_bake_plan_config(bake_plan_config):
     cfg = PlanConfig("", "", bake_plan_config)
     assert cfg.name == "armv7 validation"
     assert cfg.description == "Build and test linux kernel for armv7"
+    assert cfg.plan_file == open(bake_plan_config).read()
 
     cfg = PlanConfig("hello", "world", bake_plan_config)
     assert cfg.name == "hello"
     assert cfg.description == "world"
+    assert cfg.plan_file == open(bake_plan_config).read()
 
     assert len(cfg.plan) == 3
 
@@ -225,10 +435,12 @@ def test_bake_plan_extended_config(bake_plan_extended_config):
     cfg = PlanConfig("", "", bake_plan_extended_config)
     assert cfg.name == "OE RPB tux CI test (LTS)"
     assert cfg.description == "A sanity test for OE RPB"
+    assert cfg.plan_file == open(bake_plan_extended_config).read()
 
     cfg = PlanConfig("poky", "lts", bake_plan_extended_config)
     assert cfg.name == "poky"
     assert cfg.description == "lts"
+    assert cfg.plan_file == open(bake_plan_extended_config).read()
 
     assert len(cfg.plan) == 4
 
@@ -241,10 +453,10 @@ def test_bake_plan_extended_config(bake_plan_extended_config):
 
 
 def test_plan_config_job_name(plan_config):
-    cfg = PlanConfig("hello", "world", plan_config, "gcc-simple")
+    cfg = PlanConfig("hello", "world", plan_config, ["gcc-simple"])
     assert len(cfg.plan) == 0
 
-    cfg = PlanConfig("hello", "world", plan_config, "simple-gcc")
+    cfg = PlanConfig("hello", "world", plan_config, ["simple-gcc"])
     assert len(cfg.plan) == 1
     assert cfg.plan[0]["build"] == {
         "kconfig": "tinyconfig",
@@ -257,7 +469,7 @@ def test_bake_plan_config_job_name(bake_plan_config):
     cfg = PlanConfig("hello", "world", bake_plan_config)
     assert len(cfg.plan) == 3
 
-    cfg = PlanConfig("hello", "world", bake_plan_config, "armv7")
+    cfg = PlanConfig("hello", "world", bake_plan_config, ["armv7"])
     assert len(cfg.plan) == 1
     assert cfg.plan[0]["build"]["machine"] == "ledge-multi-armv7"
     assert not cfg.plan[0]["tests"]
@@ -265,7 +477,7 @@ def test_bake_plan_config_job_name(bake_plan_config):
 
 def test_plan_config_version(plan_config_unknown_version):
     with pytest.raises(InvalidConfiguration):
-        PlanConfig("hello", "world", plan_config_unknown_version, "gcc-simple")
+        PlanConfig("hello", "world", plan_config_unknown_version, ["gcc-simple"])
 
 
 def test_submit(config, plan_config, mocker):
@@ -278,14 +490,21 @@ def test_submit(config, plan_config, mocker):
         project="tuxprj",
         no_cache=True,
     )
+    call = 0
 
     def post(url, headers, data):
+        nonlocal call
         assert "Authorization" in headers
         if url == "https://tuxapi.tuxsuite.com/v1/groups/tuxgrp/projects/tuxprj/plans":
             assert data == {
                 "description": "A simple plan",
                 "name": "Simple plan",
                 "is_public": True,
+                "plan_callback": None,
+                "plan_callback_headers": None,
+                "count": 41,
+                "plan_file": ANY,
+                "notify_emails": [],
             }
             return {"uid": "my-plan-uid"}
         if url == "https://tuxapi.tuxsuite.com/v1/groups/tuxgrp/projects/tuxprj/builds":
@@ -307,7 +526,11 @@ def test_submit(config, plan_config, mocker):
                 d["download_url"] = f"http://url/{d['uid']}"
                 index += 1
             return builds
-        if url == "https://tuxapi.tuxsuite.com/v1/groups/tuxgrp/projects/tuxprj/tests":
+        if (
+            url == "https://tuxapi.tuxsuite.com/v1/groups/tuxgrp/projects/tuxprj/tests"
+            and call == 0
+        ):
+            call += 1
             assert isinstance(data, list)
             assert len(data) == 30
 
@@ -321,30 +544,30 @@ def test_submit(config, plan_config, mocker):
                 assert t.get("tests") == d.get("tests")
                 assert t.get("rootfs") == d.get("rootfs")
 
-            assert data[0]["waiting_for"] == "my-build-00"
-            assert data[1]["waiting_for"] == "my-build-01"
-            assert data[2]["waiting_for"] == "my-build-02"
-            assert data[3]["waiting_for"] == "my-build-03"
-            assert data[4]["waiting_for"] == "my-build-04"
-            assert data[5]["waiting_for"] == "my-build-05"
-            assert data[6]["waiting_for"] == "my-build-06"
-            assert data[7]["waiting_for"] == "my-build-07"
-            assert data[8]["waiting_for"] == "my-build-07"
-            assert data[9]["waiting_for"] == "my-build-08"
-            assert data[10]["waiting_for"] == "my-build-08"
-            assert data[11]["waiting_for"] == "my-build-08"
-            assert data[12]["waiting_for"] == "my-build-08"
-            assert data[13]["waiting_for"] == "my-build-08"
-            assert data[14]["waiting_for"] == "my-build-09"
-            assert data[15]["waiting_for"] == "my-build-09"
-            assert data[16]["waiting_for"] == "my-build-09"
-            assert data[17]["waiting_for"] == "my-build-09"
-            assert data[18]["waiting_for"] == "my-build-09"
-            assert data[19]["waiting_for"] == "my-build-10"
-            assert data[20]["waiting_for"] == "my-build-10"
-            assert data[21]["waiting_for"] == "my-build-10"
-            assert data[22]["waiting_for"] == "my-build-10"
-            assert data[23]["waiting_for"] == "my-build-10"
+            assert data[0]["waiting_for"] == "BUILD#my-build-00"
+            assert data[1]["waiting_for"] == "BUILD#my-build-01"
+            assert data[2]["waiting_for"] == "BUILD#my-build-02"
+            assert data[3]["waiting_for"] == "BUILD#my-build-03"
+            assert data[4]["waiting_for"] == "BUILD#my-build-04"
+            assert data[5]["waiting_for"] == "BUILD#my-build-05"
+            assert data[6]["waiting_for"] == "BUILD#my-build-06"
+            assert data[7]["waiting_for"] == "BUILD#my-build-07"
+            assert data[8]["waiting_for"] == "BUILD#my-build-07"
+            assert data[9]["waiting_for"] == "BUILD#my-build-08"
+            assert data[10]["waiting_for"] == "BUILD#my-build-08"
+            assert data[11]["waiting_for"] == "BUILD#my-build-08"
+            assert data[12]["waiting_for"] == "BUILD#my-build-08"
+            assert data[13]["waiting_for"] == "BUILD#my-build-08"
+            assert data[14]["waiting_for"] == "BUILD#my-build-09"
+            assert data[15]["waiting_for"] == "BUILD#my-build-09"
+            assert data[16]["waiting_for"] == "BUILD#my-build-09"
+            assert data[17]["waiting_for"] == "BUILD#my-build-09"
+            assert data[18]["waiting_for"] == "BUILD#my-build-09"
+            assert data[19]["waiting_for"] == "BUILD#my-build-10"
+            assert data[20]["waiting_for"] == "BUILD#my-build-10"
+            assert data[21]["waiting_for"] == "BUILD#my-build-10"
+            assert data[22]["waiting_for"] == "BUILD#my-build-10"
+            assert data[23]["waiting_for"] == "BUILD#my-build-10"
             for i in range(24, 30):
                 assert "waiting_for" not in data[i]
                 assert "kernel" in data[i]
@@ -364,7 +587,7 @@ def test_submit(config, plan_config, mocker):
     # batch process test
     """
     after post request, the returned list of object is used to update plan.status["builds"]
-    with tuxapi returned object along with uid and downlaod url
+    with tuxapi returned object along with uid and download url
     """
     assert len(plan.status["builds"]) == 11
     # checking updated status of builds after batch processing
@@ -374,6 +597,190 @@ def test_submit(config, plan_config, mocker):
         assert uid == f"my-build-{index:02}"
         assert status["download_url"] == f"http://url/{uid}"
         index += 1
+    tests = plan.status["tests"]
+    assert len(plan.status["tests"]) == 30
+    index = 0
+    for uid, status in tests.items():
+        assert uid == f"my-test-{index:02}"
+        index += 1
+    # check if parameters passed are set correctly in dict
+    plan = Plan(
+        cfg,
+        git_repo="https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git",
+        git_ref="master",
+        group="tuxgrp",
+        project="tuxprj",
+        no_cache=True,
+        parameters=[("a", "b"), ("c", "d")],
+    )
+    call = 0
+    plan.submit()
+    for test in plan.tests:
+        assert test.parameters["a"] == "b"
+        assert test.parameters["c"] == "d"
+
+
+def test_submit_with_sanity(config, sanity_plan_config, mocker):
+    cfg = PlanConfig("", "", sanity_plan_config)
+    plan = Plan(
+        cfg,
+        git_repo="https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git",
+        git_ref="master",
+        group="tuxgrp",
+        project="tuxprj",
+        no_cache=True,
+    )
+
+    call = 0
+    index = 0
+
+    def post(url, headers, data):
+        nonlocal call
+        nonlocal index
+        assert "Authorization" in headers
+        if url == "https://tuxapi.tuxsuite.com/v1/groups/tuxgrp/projects/tuxprj/plans":
+            assert data == {
+                "description": "A simple sanity plan",
+                "name": "Simple sanity plan",
+                "is_public": True,
+                "plan_callback": None,
+                "plan_callback_headers": None,
+                "count": 37,
+                "plan_file": ANY,
+                "notify_emails": [],
+            }
+            return {"uid": "my-plan-uid"}
+        if url == "https://tuxapi.tuxsuite.com/v1/groups/tuxgrp/projects/tuxprj/builds":
+            assert isinstance(data, dict)
+            assert len(data["builds"]) == 5
+            builds = data["builds"]
+            index = 0
+            for p, d in zip(cfg.plan, builds):
+                assert p["build"]["toolchain"] == d["toolchain"]
+                assert p["build"]["target_arch"] == d["target_arch"]
+                assert [p["build"]["kconfig"]] == d["kconfig"]
+                assert d["git_ref"] == "master"
+                assert (
+                    d["git_repo"]
+                    == "https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git"
+                )
+                assert d["plan"] == "my-plan-uid"
+                d["uid"] = f"my-build-{index:02}"
+                d["download_url"] = f"http://url/{d['uid']}"
+                index += 1
+            return builds
+        if (
+            url == "https://tuxapi.tuxsuite.com/v1/groups/tuxgrp/projects/tuxprj/tests"
+            and call == 0
+        ):
+            call += 1
+            # sanity tests only
+            assert isinstance(data, list)
+            assert len(data) == 5
+
+            tests = []
+            for p in cfg.plan:
+                if p.get("sanity_test"):
+                    tests.append(p["sanity_test"])
+
+            for t, d in zip(tests, data):
+                assert t["device"] == d["device"]
+                assert t.get("kernel") == d.get("kernel")
+                assert t.get("tests") == d.get("tests")
+                assert t.get("rootfs") == d.get("rootfs")
+
+            assert data[0]["waiting_for"] == "BUILD#my-build-00"
+            assert data[1]["waiting_for"] == "BUILD#my-build-01"
+            assert data[2]["waiting_for"] == "BUILD#my-build-02"
+            assert data[3]["waiting_for"] == "BUILD#my-build-03"
+            assert "waiting_for" not in data[4]
+
+            index = 0
+            for i, d in enumerate(data):
+                d["uid"] = f"my-test-{index:02}"
+                index += 1
+            return data
+
+        if (
+            url == "https://tuxapi.tuxsuite.com/v1/groups/tuxgrp/projects/tuxprj/tests"
+            and call == 1
+        ):
+            assert isinstance(data, list)
+            assert len(data) == 27
+
+            tests = []
+            for p in cfg.plan:
+                tests.extend(p["tests"])
+
+            for t, d in zip(tests, data):
+                assert t["device"] == d["device"]
+                assert t.get("kernel") == d.get("kernel")
+                assert t.get("tests") == d.get("tests")
+                assert t.get("rootfs") == d.get("rootfs")
+
+            assert data[0]["waiting_for"] == "TEST#my-test-00"
+            assert data[1]["waiting_for"] == "TEST#my-test-00"
+            assert data[2]["waiting_for"] == "TEST#my-test-01"
+            assert data[3]["waiting_for"] == "TEST#my-test-01"
+            assert data[4]["waiting_for"] == "TEST#my-test-01"
+            assert data[5]["waiting_for"] == "TEST#my-test-01"
+            assert data[6]["waiting_for"] == "TEST#my-test-01"
+            assert data[7]["waiting_for"] == "TEST#my-test-02"
+            assert data[8]["waiting_for"] == "TEST#my-test-02"
+            assert data[9]["waiting_for"] == "TEST#my-test-02"
+            assert data[10]["waiting_for"] == "TEST#my-test-02"
+            assert data[11]["waiting_for"] == "TEST#my-test-02"
+            assert data[12]["waiting_for"] == "TEST#my-test-03"
+            assert data[13]["waiting_for"] == "TEST#my-test-03"
+            assert data[14]["waiting_for"] == "TEST#my-test-03"
+            assert data[15]["waiting_for"] == "TEST#my-test-03"
+            assert data[16]["waiting_for"] == "TEST#my-test-03"
+            assert data[17]["waiting_for"] == "BUILD#my-build-04"
+            assert data[18]["waiting_for"] == "BUILD#my-build-04"
+            assert data[19]["waiting_for"] == "TEST#my-test-04"
+            assert data[20]["waiting_for"] == "TEST#my-test-04"
+            assert data[21]["waiting_for"] == "TEST#my-test-04"
+            assert data[22]["waiting_for"] == "TEST#my-test-04"
+            assert data[23]["waiting_for"] == "TEST#my-test-04"
+            assert data[24]["waiting_for"] == "TEST#my-test-04"
+
+            for i in range(25, 27):
+                assert "waiting_for" not in data[i]
+                assert "kernel" in data[i]
+
+            for i, d in enumerate(data):
+                d["uid"] = f"my-test-{index:02}"
+                index += 1
+            return data
+
+        assert 0
+
+    mocker.patch("tuxsuite.build.post_request", post)
+    plan.submit()
+
+    assert len(plan.builds) == 5
+    assert len(plan.tests) == 32
+
+    # batch process test
+    """
+    after post request, the returned list of object is used to update plan.status["builds"]
+    with tuxapi returned object along with uid and download url
+    """
+    assert len(plan.status["builds"]) == 5
+    # checking updated status of builds after batch processing
+    builds = plan.status["builds"]
+    index = 0
+    for uid, status in builds.items():
+        assert uid == f"my-build-{index:02}"
+        assert status["download_url"] == f"http://url/{uid}"
+        index += 1
+    assert len(plan.status["tests"]) == 32
+    # checking updated status of builds after batch processing
+    tests = plan.status["tests"]
+    index = 0
+    for uid, status in tests.items():
+        assert uid == f"my-test-{index:02}"
+        index += 1
 
 
 def test_bake_submit(config, bake_plan_config, mocker):
@@ -395,6 +802,11 @@ def test_bake_submit(config, bake_plan_config, mocker):
                 "name": "armv7 validation",
                 "description": "Build and test linux kernel for armv7",
                 "is_public": True,
+                "plan_callback": None,
+                "plan_callback_headers": None,
+                "count": 3,
+                "plan_file": ANY,
+                "notify_emails": [],
             }
             return {"uid": "my-plan-uid"}
         if (
@@ -428,7 +840,7 @@ def test_bake_submit(config, bake_plan_config, mocker):
     # batch process test
     """
     after post request, the returned list of object is used to update plan.status["builds"]
-    with tuxapi returned object along with uid and downlaod url
+    with tuxapi returned object along with uid and download url
     """
     assert len(plan.status["builds"]) == 3
     # checking updated status of builds after batch processing
@@ -459,6 +871,11 @@ def test_bake_plan_extended_config_submit(config, bake_plan_extended_config, moc
                 "name": "OE RPB tux CI test (LTS)",
                 "description": "A sanity test for OE RPB",
                 "is_public": True,
+                "plan_callback": None,
+                "plan_callback_headers": None,
+                "count": 4,
+                "plan_file": ANY,
+                "notify_emails": [],
             }
             return {"uid": "my-plan-uid"}
         if (
@@ -490,6 +907,105 @@ def test_bake_plan_extended_config_submit(config, bake_plan_extended_config, moc
     assert len(plan.tests) == 0
 
 
+def test_bake_submit_with_test(config, bake_plan_config_with_test, mocker):
+    cfg = PlanConfig("", "", bake_plan_config_with_test)
+    plan = Plan(
+        cfg,
+        group="bake_tuxgrp",
+        project="bake_tuxprj",
+    )
+
+    def post(url, headers, data):
+        assert "Authorization" in headers
+        if (
+            url
+            == "https://tuxapi.tuxsuite.com/v1/groups/bake_tuxgrp/projects/bake_tuxprj/plans"
+        ):
+            assert data == {
+                "name": "armv7 validation",
+                "description": "Build and test linux kernel for armv7",
+                "is_public": True,
+                "plan_callback": None,
+                "plan_callback_headers": None,
+                "count": 7,
+                "plan_file": ANY,
+                "notify_emails": [],
+            }
+            return {"uid": "my-plan-uid"}
+        if (
+            url
+            == "https://tuxapi.tuxsuite.com/v1/groups/bake_tuxgrp/projects/bake_tuxprj/oebuilds"
+        ):
+            assert isinstance(data, dict)
+            assert len(data["oebuilds"]) == 3
+            builds = data["oebuilds"]
+            index = 0
+            for p, d in zip(cfg.plan, builds):
+                assert p["build"]["container"] == d["container"]
+                assert p["build"]["distro"] == d["distro"]
+                assert p["build"]["envsetup"] == d["envsetup"]
+                assert p["build"]["machine"] == d["machine"]
+                assert p["build"]["sources"] == d["sources"]
+                assert p["build"]["target"] == d["target"]
+                assert d["plan"] == "my-plan-uid"
+                d["uid"] = f"my-build-{index:02}"
+                d["download_url"] = f"http://url/{d['uid']}"
+                index += 1
+            return builds
+        if (
+            url
+            == "https://tuxapi.tuxsuite.com/v1/groups/bake_tuxgrp/projects/bake_tuxprj/tests"
+        ):
+            assert isinstance(data, list)
+            assert len(data) == 4
+
+            tests = []
+            for p in cfg.plan:
+                tests.extend(p["tests"])
+
+            for t, d in zip(tests, data):
+                assert t["device"] == d["device"]
+                assert t.get("kernel") == d.get("kernel")
+                assert t.get("tests") == d.get("tests")
+                assert t.get("rootfs") == d.get("rootfs")
+
+            assert data[0]["waiting_for"] == "OEBUILD#my-build-01"
+            assert data[1]["waiting_for"] == "OEBUILD#my-build-01"
+            assert data[2]["waiting_for"] == "OEBUILD#my-build-02"
+            assert data[3]["waiting_for"] == "OEBUILD#my-build-02"
+
+            for i, d in enumerate(data):
+                d["uid"] = f"my-test-{i:02}"
+            return data
+
+        assert 0
+
+    mocker.patch("tuxsuite.build.post_request", post)
+    plan.submit()
+    assert len(plan.builds) == 3
+    assert len(plan.tests) == 4
+
+    # batch process test
+    """
+    after post request, the returned list of object is used to update plan.status["builds"]
+    with tuxapi returned object along with uid and download url
+    """
+    assert len(plan.status["builds"]) == 3
+    # checking updated status of builds after batch processing
+    builds = plan.status["builds"]
+    index = 0
+    for uid, status in builds.items():
+        assert uid == f"my-build-{index:02}"
+        assert status["download_url"] == f"http://url/{uid}"
+        index += 1
+    tests = plan.status["tests"]
+    assert len(plan.status["tests"]) == 4
+    index = 0
+    for uid, status in tests.items():
+        assert uid == f"my-test-{index:02}"
+        index += 1
+
+
 def test_get_plan(config, plan_config, mocker):
     cfg = PlanConfig("", "", plan_config)
     plan = Plan(
@@ -700,6 +1216,8 @@ def test_watch(config, mocker, plan_config):
         "token": "test_token",
         "tuxapi_url": "http://tuxapi",
         "kbapi_url": "http://test/foo",
+        "lava_test_plans_project": None,
+        "lab": None,
         "device": "qemu-armv7",
         "kernel": "",
     }
