@@ -24,8 +24,3 @@ def test_new_build(config, params):
     build = tuxsuite.Build(**params)
     assert build.token is not None
     assert build.kbapi_url is not None
-
-
-def test_new_build_set(config, params):
-    build_set = tuxsuite.BuildSet([params])
-    assert build_set.tuxapi_url is not None
