@@ -15,17 +15,16 @@ import subprocess
 import tarfile
 import tempfile
 import time
+import yaml
+import xml.etree.ElementTree as ET
 import tuxsuite.exceptions
 import tuxsuite.requests
 from tuxsuite import __version__
 import uuid
 from urllib.parse import urlparse
+from tuxsuite.cli import error
 
 
-# We will poll for status change for an average duration of 240 minutes for builds
-build_state_timeout = 14400  # 60 * 240
-# We will poll for status change for an average duration of 300 minutes for tests
-test_state_timeout = 18000  # 60 * 300
 delay_status_update = int(os.getenv("TUXSUITE_DELAY_STATUS_UPDATE", "29")) + 1
 
 
@@ -164,26 +163,42 @@ def handle_patch(patch_path):
     return (patch_file, patch_content.decode("utf-8"))
 
 
-def handle_manifest(manifest_path):
-    manifest_is_web_url = False
-    manifest_content = None
-    manifest_file = os.path.basename(manifest_path)
-    if urlparse(manifest_path).scheme in ["http", "https"]:
-        manifest_file = manifest_path
-        manifest_content = b""
-        manifest_is_web_url = True
+def handle_attachment(file_path, file_type="xml"):
+    file_is_web_url = False
+    file_content = None
+    filename = os.path.basename(file_path)
+    if urlparse(file_path).scheme in ["http", "https"]:
+        filename = file_path
+        file_content = b""
+        file_is_web_url = True
     else:
-        if os.path.isfile(manifest_path):
-            file_type, file_encoding = mimetypes.guess_type(manifest_path)
-            if file_type not in ["application/xml", "text/xml"]:
-                raise Exception("Manifest format unknown")
-            with open(manifest_path, "rb") as manifest:
-                manifest_content = base64.b64encode(manifest.read())
-
-    if not manifest_is_web_url:
-        manifest_sha256 = hashlib.sha256(manifest_content).hexdigest()
-        manifest_file = f"{manifest_sha256}/{manifest_file}"
-    return (manifest_file, manifest_content.decode("utf-8"))
+        ext = os.path.splitext(file_path)[1].lower()
+        if os.path.isfile(file_path) and ext in [".xml", ".yml", ".yaml"]:
+            if file_type == "yaml" and ext not in [".yml", ".yaml"]:
+                error(f"{file_path} does not have yaml file extension")
+            elif file_type == "xml" and ext != ".xml":
+                error(f"{file_path} does not have xml file extension")
+        else:
+            error(f"{file_path}: Is provided file a valid xml/yaml file?")
+        # extension of file
+        with open(file_path, "r") as f:
+            try:
+                if ext == ".xml":
+                    # parse XML file
+                    ET.parse(f)
+                elif ext in [".yml", ".yaml"]:
+                    # parse yaml file
+                    yaml.safe_load(f)
+            except ET.ParseError as e:
+                error(f"Error parsing {file_path} file: {e}")
+            except yaml.YAMLError as e:
+                error(f"Error parsing {file_path} file: {e}")
+        with open(file_path, "rb") as infile:
+            file_content = base64.b64encode(infile.read())
+    if not file_is_web_url:
+        file_sha256 = hashlib.sha256(file_content).hexdigest()
+        filename = f"{file_sha256}/{filename}"
+    return (filename, file_content.decode("utf-8"))
 
 
 class HeadersMixin:
@@ -238,6 +253,8 @@ class Build(HeadersMixin):
     no_cache = attrib(default=False)
     is_public = attrib(default=True)
     callback = attrib(default=None)
+    callback_headers = attrib(default=None)
+    notify_emails = attrib(default=[])
 
     def __attrs_post_init__(self):
         if isinstance(self.kconfig, str):
@@ -252,7 +269,7 @@ class Build(HeadersMixin):
         git_short_log = self.status.get("git_short_log", "")
 
         if len(self.kconfig) > 1:
-            kconfig = f"{self.kconfig[0]}+{len(self.kconfig)-1}"
+            kconfig = f"{self.kconfig[0]}+{len(self.kconfig)-1}"  # noqa: E226
         else:
             kconfig = self.kconfig[0]
         return "{} {} ({}) with {} @ {}".format(
@@ -307,21 +324,28 @@ class Build(HeadersMixin):
             patch[build_entry["kernel_patch_file"]] = kernel_patch
         if self.callback is not None:
             build_entry["callback"] = self.callback
+        if self.callback_headers is not None:
+            build_entry["callback_headers"] = self.callback_headers
+        if self.notify_emails:
+            build_entry["notify_emails"] = self.notify_emails
         return (build_entry, patch)
 
     def verify_build_parameters(self):
         """Pre-check the build arguments"""
-        assert self.git_repo, "git_repo is mandatory\n"
-        assert self.is_supported_git_url(
-            self.git_repo
-        ), "git url must be in the form of git:// or http:// or https://\n"
-        assert (self.git_ref and not self.git_sha) or (
-            self.git_sha and not self.git_ref
-        ), "Either a git_ref or a git_sha is required\n"
-        assert self.target_arch is not None, "target-arch is mandatory\n"
-        assert self.kconfig, "kconfig is mandatory\n"
-        assert self.toolchain, "toolchain is mandatory\n"
-        assert self.headers is not None, "headers is mandatory\n"
+        try:
+            assert self.git_repo, "git_repo is mandatory\n"
+            assert self.is_supported_git_url(
+                self.git_repo
+            ), "git url must be in the form of git:// or http:// or https://\n"
+            assert (self.git_ref and not self.git_sha) or (
+                self.git_sha and not self.git_ref
+            ), "Either a git_ref or a git_sha is required\n"
+            assert self.target_arch is not None, "target-arch is mandatory\n"
+            assert self.kconfig, "kconfig is mandatory\n"
+            assert self.toolchain, "toolchain is mandatory\n"
+            assert self.headers is not None, "headers is mandatory\n"
+        except AssertionError as exc:
+            error(str(exc))
 
     def build(self, plan=None):
         """Submit the build request"""
@@ -347,8 +371,6 @@ class Build(HeadersMixin):
         """
         Wait for the build to finish. Whenever the state changes, yielding the
         new state).
-
-        Will timeout after `build_state_timeout` seconds.
         """
         waiting_states = {
             "queued": BuildState(
@@ -380,7 +402,6 @@ class Build(HeadersMixin):
                 cli_color="cyan",
             ),
         }
-        timeout = time.time() + build_state_timeout
         previous_state = None
         while True:
             self.get_status()
@@ -389,7 +410,6 @@ class Build(HeadersMixin):
             if state != previous_state:
                 if retry:
                     message = self.status.get("status_message", "infrastructure error")
-                    timeout = time.time() + build_state_timeout
                     yield BuildState(
                         build=self,
                         state=self.status["tuxbuild_status"],
@@ -410,10 +430,6 @@ class Build(HeadersMixin):
                         icon="🎰",
                         cli_color="yellow",
                     )
-            elif time.time() >= timeout:
-                raise tuxsuite.exceptions.Timeout(
-                    f"Build timed out after {build_state_timeout/60} minutes; abandoning"
-                )
             elif iterator:
                 yield
 
@@ -521,9 +537,15 @@ class Plan(HeadersMixin):
         self.tuxapi_url = kwargs["tuxapi_url"]
         self.project = kwargs["project"]
         self.group = kwargs["group"]
+        self.lava_test_plans_project = kwargs["lava_test_plans_project"]
+        self.lab = kwargs["lab"]
         self.args = kwargs
+        self.parameters = kwargs.get("parameters", [])
         self.plan = kwargs.get("plan", None)
         self.is_public = kwargs.get("is_public", True)  # plan public or private
+        self.plan_callback = kwargs.get("plan_callback")
+        self.plan_callback_headers = kwargs.get("plan_callback_headers")
+        self.notify_emails = kwargs.get("notify_emails", [])
         self.builds = []
         self.tests = []
         self.status = {}
@@ -541,6 +563,7 @@ class Plan(HeadersMixin):
     def submit(self):
         builds = []
         tests = []
+        sanity_tests = []
         builds_status = {}
         tests_status = {}
 
@@ -552,68 +575,61 @@ class Plan(HeadersMixin):
                 "name": self.config.name,
                 "description": self.config.description,
                 "is_public": self.is_public,
+                "plan_callback": self.plan_callback,
+                "plan_callback_headers": self.plan_callback_headers,
+                "count": self.plan_type.count(self),
+                "plan_file": self.config.plan_file,
+                "notify_emails": self.notify_emails,
             },
         )["uid"]
 
         # calling respective plan type class to create builds
         self.plan_type.create_builds(self, builds)
 
-        # Create the tests
+        # create sanity tests
         for build, cfg in zip(builds, self.config.plan):
+            if cfg.get("sanity_test"):
+                sanity_tests.append(
+                    self.get_test_obj(build if build else None, cfg["sanity_test"])
+                )
+            else:
+                sanity_tests.append(None)
+
+        # send sanity_test to tuxapi
+        filtered_sanity_tests = [t for t in sanity_tests if t]
+        if filtered_sanity_tests:
+            ret = post_request(
+                f"{self.url}/tests",
+                self.headers,
+                [
+                    t.generate_test_request(plan=self.plan)
+                    for t in filtered_sanity_tests
+                ],
+            )
+            for test, data in zip(filtered_sanity_tests, ret):
+                test.uid = data["uid"]
+                test.status = data
+                self.tests.append(test)
+
+        # Create the tests
+        for build, cfg, sanity_test in zip(builds, self.config.plan, sanity_tests):
             if build:
                 for test in cfg["tests"]:
                     tests.append(
-                        Test(
-                            token=self.token,
-                            kbapi_url="",
-                            tuxapi_url=self.tuxapi_url,
-                            group=self.group,
-                            project=self.project,
-                            bios=test.get("bios"),
-                            boot_args=test.get("boot_args"),
-                            device=test["device"],
-                            dtb=test.get("dtb"),
-                            parameters=test.get("parameters", {}),
-                            rootfs=test.get("rootfs"),
-                            tests=test.get("tests", []),
-                            timeouts=test.get("timeouts"),
-                            callback=test.get("callback"),
-                            wait_for=build.uid,
-                            commands=test.get("commands", []),
-                            qemu_image=test.get("qemu_image"),
-                            tuxbuild=test.get("tuxbuild"),
+                        self.get_test_obj(
+                            build,
+                            test,
+                            f"TEST#{sanity_test.uid}" if sanity_test else None,
                         )
                     )
                 self.builds.append(build)
             else:
                 for test in cfg["tests"]:
                     tests.append(
-                        Test(
-                            token=self.token,
-                            kbapi_url="",
-                            tuxapi_url=self.tuxapi_url,
-                            group=self.group,
-                            project=self.project,
-                            ap_romfw=test.get("ap_romfw"),
-                            bios=test.get("bios"),
-                            boot_args=test.get("boot_args"),
-                            device=test["device"],
-                            dtb=test.get("dtb"),
-                            fip=test.get("fip"),
-                            kernel=test.get("kernel"),
-                            mcp_fw=test.get("mcp_fw"),
-                            mcp_romfw=test.get("mcp_romfw"),
-                            modules=test.get("modules"),
-                            parameters=test.get("parameters", {}),
-                            rootfs=test.get("rootfs"),
-                            scp_fw=test.get("scp_fw"),
-                            scp_romfw=test.get("scp_romfw"),
-                            tests=test.get("tests", []),
-                            timeouts=test.get("timeouts"),
-                            callback=test.get("callback"),
-                            commands=test.get("commands", []),
-                            qemu_image=test.get("qemu_image"),
-                            tuxbuild=test.get("tuxbuild"),
+                        self.get_test_obj(
+                            None,
+                            test,
+                            f"TEST#{sanity_test.uid}" if sanity_test else None,
                         )
                     )
 
@@ -641,6 +657,103 @@ class Plan(HeadersMixin):
             tests_status.update({test.uid: test.status})
         self.status = {"builds": builds_status, "tests": tests_status}
 
+    def get_test_obj(self, build, test, wait_for=None):
+        new_overlays = []
+        for overlay in test.get("overlay", []):
+            if isinstance(overlay, list):
+                ovl_len = len(overlay)
+                if ovl_len == 1:
+                    new_overlays.append([overlay[0], "/"])
+                elif ovl_len == 2:
+                    new_overlays.append([overlay[0], overlay[1]])
+                if ovl_len > 2:
+                    raise Exception(
+                        f"overlay can't have more than 2 parameters: {overlay}"
+                    )
+            else:
+                new_overlays.append([overlay, "/"])
+        test["overlay"] = new_overlays
+        # process additional parameters if available in cmdargs
+        test["parameters"] = test.get("parameters", {})
+        for p in self.parameters:
+            key, val = p
+            if key not in test["parameters"]:
+                test["parameters"][key] = val
+        if build:
+            return Test(
+                token=self.token,
+                kbapi_url="",
+                tuxapi_url=self.tuxapi_url,
+                group=self.group,
+                project=self.project,
+                lava_test_plans_project=test.get("lava_test_plans_project")
+                or self.lava_test_plans_project,
+                lab=test.get("lab") or self.lab,
+                bios=test.get("bios"),
+                boot_args=test.get("boot_args"),
+                device=test["device"],
+                dtb=test.get("dtb"),
+                parameters=test.get("parameters", {}),
+                rootfs=test.get("rootfs"),
+                tests=test.get("tests", []),
+                test_name=test.get("test_name", None),
+                overlays=test.get("overlay", []),
+                timeouts=test.get("timeouts"),
+                callback=test.get("callback") or self.args.get("callback"),
+                callback_headers=test.get("callback_header")
+                or self.args.get("callback_headers"),
+                wait_for=(
+                    f"{self.plan_type.name}#{build.uid}"
+                    if wait_for is None
+                    else wait_for
+                ),
+                commands=test.get("commands", []),
+                qemu_image=test.get("qemu_image"),
+                shared=test.get("shared"),
+                host=test.get("host"),
+                tuxbuild=test.get("tuxbuild"),
+                notify_emails=test.get("notify_emails"),
+            )
+        else:
+            return Test(
+                token=self.token,
+                kbapi_url="",
+                tuxapi_url=self.tuxapi_url,
+                group=self.group,
+                project=self.project,
+                lava_test_plans_project=test.get("lava_test_plans_project")
+                or self.lava_test_plans_project,
+                lab=test.get("lab") or self.lab,
+                ap_romfw=test.get("ap_romfw"),
+                bios=test.get("bios"),
+                boot_args=test.get("boot_args"),
+                device=test["device"],
+                dtb=test.get("dtb"),
+                fip=test.get("fip"),
+                kernel=test.get("kernel"),
+                mcp_fw=test.get("mcp_fw"),
+                mcp_romfw=test.get("mcp_romfw"),
+                modules=test.get("modules"),
+                overlays=test.get("overlay", []),
+                parameters=test.get("parameters", {}),
+                rootfs=test.get("rootfs"),
+                scp_fw=test.get("scp_fw"),
+                scp_romfw=test.get("scp_romfw"),
+                tests=test.get("tests", []),
+                test_name=test.get("test_name", None),
+                timeouts=test.get("timeouts"),
+                callback=test.get("callback") or self.args.get("callback"),
+                callback_headers=test.get("callback_header")
+                or self.args.get("callback_headers"),
+                commands=test.get("commands", []),
+                qemu_image=test.get("qemu_image"),
+                shared=test.get("shared"),
+                host=test.get("host"),
+                tuxbuild=test.get("tuxbuild"),
+                notify_emails=test.get("notify_emails"),
+                wait_for=wait_for,
+            )
+
     def load_kernel_builds(self, b):
         git_ref = b.get("git_ref")
         git_sha = b.get("git_sha")
@@ -721,6 +834,8 @@ class Plan(HeadersMixin):
                 tuxapi_url=self.tuxapi_url,
                 group=self.group,
                 project=t["project"],
+                lava_test_plans_project=self.lava_test_plans_project,
+                lab=self.lab,
                 status=t,
                 tests=t["tests"],
                 uid=t["uid"],
@@ -763,7 +878,11 @@ class Plan(HeadersMixin):
         return self.status
 
     def _tests_wait_for(self, uid):
-        return [t for t in self.tests if t.status["waiting_for"] == uid]
+        return [
+            t
+            for t in self.tests
+            if t.status["waiting_for"] and t.status["waiting_for"].split("#")[1] == uid
+        ]
 
     def filter_builds(self, f):
         return sorted([b for b in self.builds if f(self, b)])
@@ -885,6 +1004,8 @@ class Test(HeadersMixin):
     tuxapi_url = attrib()
     group = attrib()
     project = attrib()
+    lava_test_plans_project = attrib()
+    lab = attrib()
     device = attrib()
 
     ap_romfw = attrib(default=None)
@@ -892,25 +1013,33 @@ class Test(HeadersMixin):
     bios = attrib(default=None)
     dtb = attrib(default=None)
     fip = attrib(default=None)
+    job_definition = attrib(default=None)
     kernel = attrib(default=None)
     mcp_fw = attrib(default=None)
     mcp_romfw = attrib(default=None)
     modules = attrib(default=None)
+    overlays = attrib(default=[])
     parameters = attrib(default=[])
     rootfs = attrib(default=None)
     scp_fw = attrib(default=None)
     scp_romfw = attrib(default=None)
     tests = attrib(default=[])
+    test_name = attrib(default=None)
     timeouts = attrib(default=[])
     callback = attrib(default=None)
+    callback_headers = attrib(default=None)
     commands = attrib(default=[])
     qemu_image = attrib(default=None)
+    shared = attrib(default=False)
+    host = attrib(default="x86_64")
     tuxbuild = attrib(default=None)
+    notify_emails = attrib(default=[])
 
     uid = attrib(default=None)
     wait_for = attrib(default=None)
     status = attrib(default={})
     url = attrib(default=None)
+    is_public = attrib(default=True)
 
     def __str__(self):
         tests = "[" + ", ".join(["boot"] + self.tests) + "]"
@@ -933,11 +1062,18 @@ class Test(HeadersMixin):
             "rootfs": self.rootfs,
             "scp_fw": self.scp_fw,
             "scp_romfw": self.scp_romfw,
+            "lava_test_plans_project": self.lava_test_plans_project,
+            "lab": self.lab,
+            "is_public": self.is_public,
         }
         if self.modules:
             data["modules"] = self.modules
+        if self.overlays:
+            data["overlays"] = self.overlays
         if self.tests:
             data["tests"] = self.tests
+        if self.test_name:
+            data["test_name"] = self.test_name
         if self.timeouts:
             data["timeouts"] = self.timeouts
         if self.boot_args:
@@ -946,15 +1082,29 @@ class Test(HeadersMixin):
             data["waiting_for"] = self.wait_for
         if self.callback:
             data["callback"] = self.callback
+        if self.callback_headers:
+            data["callback_headers"] = self.callback_headers
         if self.commands:
             data["commands"] = self.commands
         if self.qemu_image:
             data["qemu_image"] = self.qemu_image
+        if self.shared:
+            data["shared"] = self.shared
+        if self.host:
+            data["host"] = self.host
         if self.tuxbuild:
             data["tuxbuild"] = self.tuxbuild
+        if self.notify_emails:
+            data["notify_emails"] = self.notify_emails
         if plan is not None:
             data["plan"] = plan
 
+        # lava job definition file
+        if self.job_definition:
+            (data["job_definition"], job_definition_content) = handle_attachment(
+                self.job_definition, file_type="yaml"
+            )
+            data["attachments"] = {data["job_definition"]: job_definition_content}
         return data
 
     def test(self, plan=None):
@@ -974,8 +1124,6 @@ class Test(HeadersMixin):
         """
         Wait for the test to finish. Whenever the state changes, yielding the
         new state).
-
-        Will timeout after `test_state_timeout` seconds.
         """
         waiting_states = {
             "waiting": BuildState(
@@ -1000,7 +1148,6 @@ class Test(HeadersMixin):
                 cli_color="cyan",
             ),
         }
-        timeout = time.time() + test_state_timeout
         previous_state = None
         while True:
             self.status = self.get()
@@ -1012,10 +1159,6 @@ class Test(HeadersMixin):
                     break
                 else:
                     assert 0
-            elif time.time() >= timeout:
-                raise tuxsuite.exceptions.Timeout(
-                    f"Test timed out after {test_state_timeout/60} minutes; abandoning"
-                )
             elif iterator:
                 yield
 
@@ -1051,7 +1194,7 @@ class Test(HeadersMixin):
         else:
             icon = "🔧"
             cli_color = "bright_red"
-            message = "TuxTest error"
+            message = self.status.get("status_message", "TuxTest error")
 
         finished = BuildState(
             build=self,
@@ -1079,6 +1222,8 @@ class Results(HeadersMixin):
     tuxapi_url = attrib()
     group = attrib()
     project = attrib()
+    lava_test_plans_project = attrib()
+    lab = attrib()
     uid = attrib(default=None)
 
     def get_build(self):
@@ -1110,6 +1255,8 @@ class Results(HeadersMixin):
             project=self.project,
             group=self.group,
             plan=self.uid,
+            lava_test_plans_project=self.lava_test_plans_project,
+            lab=self.lab,
         )
         # TODO: Return the test url as part of the test result in tuxapi
         tuxapi_url = plan.url
@@ -1161,8 +1308,11 @@ class Bitbake(HeadersMixin):
         no_cache = attrib(default=False)
         extraconfigs = attrib(default=[])
         pinned_manifest = attrib(default=None)
+        kas_override = attrib(default=None)
         is_public = attrib(default=True)
         callback = attrib(default=None)
+        callback_headers = attrib(default=None)
+        notify_emails = attrib(default=[])
 
     def __attrs_post_init__(self):
         self.build_definition = self.BuildDefinition(**(self.data))
@@ -1202,18 +1352,24 @@ class Bitbake(HeadersMixin):
         if plan is None:
             # local manifest
             if self.build_definition.manifest_file:
-                (build_data["manifest_file"], manifest) = handle_manifest(
+                (build_data["manifest_file"], manifest) = handle_attachment(
                     self.build_definition.manifest_file
                 )
                 manifests[build_data["manifest_file"]] = manifest
             # pinned manifest
             if self.build_definition.pinned_manifest:
-                (build_data["pinned_manifest"], manifest) = handle_manifest(
+                (build_data["pinned_manifest"], manifest) = handle_attachment(
                     self.build_definition.pinned_manifest
                 )
                 manifests[build_data["pinned_manifest"]] = manifest
+            # kas override
+            if self.build_definition.kas_override:
+                (build_data["kas_override"], override_content) = handle_attachment(
+                    self.build_definition.kas_override, file_type="yaml"
+                )
+                manifests[build_data["kas_override"]] = override_content
         else:
-            # we are seperately handling manifest for plan
+            # we are separately handling manifest for plan
             build_data["plan"] = plan
         return (build_data, manifests)
 
@@ -1241,8 +1397,6 @@ class Bitbake(HeadersMixin):
         """
         Wait for the build to finish. Whenever the state changes, yielding the
         new state).
-
-        Will timeout after `build_state_timeout` seconds.
         """
         waiting_states = {
             "provisioning": BuildState(
