@@ -4,25 +4,29 @@ import tuxsuite
 import tuxsuite.exceptions
 import tuxsuite.cli.colors as colors
 import tuxsuite.cli.icons as icons
+import tuxsuite.requests
+from tuxsuite.utils import error
 from tuxsuite.cli.common import common_options
 from tuxsuite.cli.models import Build
-from tuxsuite.cli.requests import apiurl, get, post, headers
+from tuxsuite.cli.requests import apiurl, get, post, headers, get_storage
 from tuxsuite.cli.utils import (
     LIMIT,
     datediff,
     fetch_next,
     key_value,
     wait_for_object,
-    error,
     format_result,
     show_log,
-    is_url,
+    get_cb_headers,
+    validate_notify_emails,
 )
 
+import argparse
 import json
 import requests
 import sys
 import time
+import pathlib
 
 
 def get_make_targets_vars(targets):
@@ -66,13 +70,32 @@ def handle_get(options, _, config):
     build = Build.new(**ret.json())
     if options.json:
         print(build.as_json())
+    elif options.json_out:
+        options.json_out.write(json.dumps(build.as_dict(), indent=4))
+    elif options.list_artifacts:
+        path_url = (
+            f"{build.download_url}{options.list_artifacts}/"
+            if options.list_artifacts is not True
+            else build.download_url
+        )
+        ret = get_storage(config, path_url).json()
+        for folder in ret.get("folders", []):
+            print(f"📁 {folder['Url'].split('/')[-2]}")
+        for file in ret.get("files", []):
+            print(f"📄 {file['Url'].split('/')[-1]}")
+        if options.download_artifacts:
+            tuxsuite.download.download_artifacts(
+                config, build, options.download_artifacts, options.list_artifacts, ret
+            )
+    elif options.download_artifacts:
+        tuxsuite.download.download_artifacts(config, build, options.download_artifacts)
     else:
         print(f"url      : {apiurl(config, build.url())}")
         print(f"project  : {build.project}")
         print(f"uid      : {build.uid}")
         print(f"plan     : {build.plan}")
         if build.waited_by:
-            print(f"tests    : {', '.join(build.waited_by)}")
+            print(f"tests    : {', '.join([t.split('#')[1] for t in build.waited_by])}")
         print(f"user     : {build.user}")
 
         print(f"kconfig  : {', '.join(build.kconfig)}")
@@ -142,28 +165,56 @@ def handle_list(options, _, config):
     if ret.status_code != 200:
         raise NotImplementedError()
 
-    builds = [Build.new(**b) for b in ret.json()["results"][: options.limit]]
+    builds = [Build.new(**b) for b in ret.json()["results"]]
     n_token = ret.json()["next"]
-    if options.json:
-        print(json.dumps([b.as_dict() for b in builds]))
+    if options.json or options.json_out:
+        itrs = options.limit // LIMIT
+        for _ in range(itrs):
+            if not n_token:
+                break
+            items, n_token = fetch_next(Build, config, url, n_token)
+            builds.extend(items)
+        if options.json:
+            print(json.dumps([b.as_dict() for b in builds[: options.limit]]))
+        elif options.json_out:
+            options.json_out.write(
+                json.dumps([b.as_dict() for b in builds[: options.limit]], indent=4)
+            )
     else:
+        previous_pt = None
         while True:
-            previous_pt = None
-            for build in builds:
+            while len(builds) < options.limit and n_token:
+                items, n_token = fetch_next(Build, config, url, n_token)
+                builds.extend(items)
+
+            if not builds:
+                break
+
+            for build in builds[: options.limit]:
                 state = build.result if build.state == "finished" else build.state
-                state_msg = (
-                    f"{colors.state(build.state, build.result)}{state}{colors.reset}"
-                )
-                warnings = (
-                    f" {colors.yellow}warnings={build.warnings_count}{colors.reset}"
-                    if build.warnings_count
-                    else ""
-                )
-                errors = (
-                    f" {colors.red}errors={build.errors_count}{colors.reset}"
-                    if build.errors_count
-                    else ""
-                )
+                if sys.stdout.isatty():
+                    state_msg = f"{colors.state(build.state, build.result)}{state}{colors.reset}"
+                    warnings = (
+                        f" {colors.yellow}warnings={build.warnings_count}{colors.reset}"
+                        if build.warnings_count
+                        else ""
+                    )
+                    errors = (
+                        f" {colors.red}errors={build.errors_count}{colors.reset}"
+                        if build.errors_count
+                        else ""
+                    )
+                else:
+                    state_msg = state
+                    warnings = (
+                        f" warnings={build.warnings_count}"
+                        if build.warnings_count
+                        else ""
+                    )
+                    errors = (
+                        f" errors={build.errors_count}" if build.errors_count else ""
+                    )
+
                 pt = build.provisioning_time
                 if pt is None:
                     pt = "....-..-..T..:..:........."
@@ -174,9 +225,14 @@ def handle_list(options, _, config):
                 )
 
                 previous_pt = pt
+            builds = builds[options.limit :]
             if sys.stdout.isatty():
-                # fetch next list of builds
-                builds, n_token = fetch_next(Build, config, url, n_token, options.limit)
+                try:
+                    input(
+                        "\nPress [ENTER] to see next list of builds, or Ctrl-C to quit:\n"
+                    )
+                except KeyboardInterrupt:
+                    sys.exit(0)
     return 0
 
 
@@ -219,7 +275,9 @@ def handle_wait(options, _, config):
             print(f"uid      : {build.uid}")
             print(f"plan     : {build.plan}")
             if build.waited_by:
-                print(f"tests    : {', '.join(build.waited_by)}")
+                print(
+                    f"tests    : {', '.join([t.split('#')[1] for t in build.waited_by])}"
+                )
             print(f"user     : {build.user}")
 
             print(f"kconfig  : {', '.join(build.kconfig)}")
@@ -298,6 +356,15 @@ def handle_submit(cmdargs, _, config):
     for e in cmdargs.environment:
         environment[e[0][0]] = e[0][1]
 
+    cb_headers = None
+    if cmdargs.callback_header:
+        if not cmdargs.callback:
+            error("--callback-header given without a --callback")
+        else:
+            cb_headers = get_cb_headers(cmdargs.callback_header)
+
+    validate_notify_emails(cmdargs.notify_email)
+
     try:
         build = tuxsuite.Build(
             git_repo=cmdargs.git_repo,
@@ -316,6 +383,8 @@ def handle_submit(cmdargs, _, config):
             no_cache=cmdargs.no_cache,
             is_public=cmdargs.private,
             callback=cmdargs.callback,
+            callback_headers=cb_headers,
+            notify_emails=cmdargs.notify_email,
         )
 
     except (AssertionError, tuxsuite.exceptions.TuxSuiteError) as e:
@@ -336,14 +405,11 @@ def handle_submit(cmdargs, _, config):
     if cmdargs.no_wait:
         format_result(build.status, tuxapi_url=build.build_data)
     else:
-        try:
-            build_result = wait_for_object(build)
-        except tuxsuite.exceptions.Timeout as exc:
-            build_result = False
-            sys.stderr.write(f"Error: {str(exc)}\n")
+        build_result = wait_for_object(build)
 
-    if cmdargs.json_out:
-        cmdargs.json_out.write(json.dumps(build.status, sort_keys=True, indent=4))
+    if cmdargs.json_out and build.status:
+        with open(cmdargs.json_out, "w") as json_out:
+            json_out.write(json.dumps(build.status, sort_keys=True, indent=4))
     if cmdargs.download:
         tuxsuite.download.download(build, cmdargs.output_dir)
     if cmdargs.show_logs:
@@ -423,7 +489,7 @@ def build_cmd_options(sp):
         "--quiet",
         default=False,
         action="store_true",
-        help="Supress all informational output; prints only the download URL for the build",
+        help="Suppress all informational output; prints only the download URL for the build",
     )
     sp.add_argument(
         "--toolchain",
@@ -450,16 +516,6 @@ def build_cmd_options(sp):
         default=None,
         help="Pin the container image sha (64 hexadecimal digits)",
     )
-    sp.add_argument(
-        "--callback",
-        default=None,
-        help=(
-            "Callback URL. The kernel build backend will send a POST "
-            "request to this URL with signed data, when the kernel "
-            "build completes."
-        ),
-        type=is_url,
-    )
 
 
 def setup_parser(parser):
@@ -470,11 +526,42 @@ def setup_parser(parser):
     # "build get <uid>"
     p = parser.add_parser("get")
     p.add_argument("uid")
-    p.add_argument("--json", action="store_true")
+    grp = p.add_mutually_exclusive_group()
+    grp.add_argument("--json", action="store_true", help="Output json build to stdout")
+    grp.add_argument(
+        "--json-out",
+        help="Write json build out to a named file path",
+        type=argparse.FileType("w", encoding="utf-8"),
+    )
+    p.add_argument(
+        "-l",
+        "--list-artifacts",
+        nargs="?",
+        default=False,
+        const=True,
+        help="List the build artifacts(files and directories) with maxdepth=1. Path (optional) can be passed "
+        "to list the build artifacts from specified path",
+    )
+    p.add_argument(
+        "-d",
+        "--download-artifacts",
+        type=pathlib.Path,
+        nargs="?",
+        const=".",
+        help="Download the build artifacts(files only) with maxdepth=1. Path (optional) can be passed "
+        "to download the build artifacts at specified location. (default: current directory)",
+    )
 
     # "build list"
     p = parser.add_parser("list")
-    p.add_argument("--json", action="store_true")
+    p.add_argument(
+        "--json", action="store_true", help="Output json builds list to stdout"
+    )
+    p.add_argument(
+        "--json-out",
+        help="Write json builds list out to a named file path",
+        type=argparse.FileType("w", encoding="utf-8"),
+    )
     p.add_argument("--limit", type=int, default=LIMIT)
 
     # "build logs <uid>"
