@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+import argparse
 import csv
 import json
 import sys
@@ -50,6 +51,8 @@ def handle_bills(options, _, config):
             )
     elif options.json:
         print(json.dumps(ret.json()))
+    elif options.json_out:
+        options.json_out.write(json.dumps(ret.json(), indent=4))
     else:
         print("date\t\tbuilds\toe\tplans\ttests\tbuilds\toe\ttests")
         for bill in bills:
@@ -73,6 +76,8 @@ def handle_get(options, _, config):
 
     if options.json:
         print(json.dumps(ret.json()))
+    elif options.json_out:
+        options.json_out.write(json.dumps(ret.json(), indent=4))
     else:
 
         def tripplet(d):
@@ -99,6 +104,8 @@ def handle_list(options, _, config):
 
     if options.json:
         print(json.dumps(ret.json()))
+    elif options.json_out:
+        options.json_out.write(json.dumps(ret.json(), indent=4))
     else:
         print("groups:")
         for grp in ret.json()["results"]:
@@ -118,15 +125,30 @@ def setup_parser(parser):
     p = parser.add_parser("bills")
     p.add_argument("name")
     p.add_argument("--csv", action="store_true")
-    p.add_argument("--json", action="store_true")
-
+    p.add_argument("--json", action="store_true", help="Output json bills to stdout")
+    p.add_argument(
+        "--json-out",
+        help="Write json bills out to a named file path",
+        type=argparse.FileType("w", encoding="utf-8"),
+    )
     # "group get <name>"
     p = parser.add_parser("get")
     p.add_argument("name")
-    p.add_argument("--json", action="store_true")
-
+    p.add_argument("--json", action="store_true", help="Output json group to stdout")
+    p.add_argument(
+        "--json-out",
+        help="Write json group out to a named file path",
+        type=argparse.FileType("w", encoding="utf-8"),
+    )
     # "group list"
     p = parser.add_parser("list")
-    p.add_argument("--json", action="store_true")
+    p.add_argument(
+        "--json", action="store_true", help="Output json group list to stdout"
+    )
+    p.add_argument(
+        "--json-out",
+        help="Write json group list out to a named file path",
+        type=argparse.FileType("w", encoding="utf-8"),
+    )
 
     return sorted(parser._name_parser_map.keys())
