@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+import argparse
 import json
 
 from tuxsuite.cli.requests import get, post
@@ -21,6 +22,8 @@ def handle_get(options, _, config):
 
     if options.json:
         print(json.dumps(ret.json()))
+    elif options.json_out:
+        options.json_out.write(json.dumps(ret.json(), indent=4))
     else:
 
         def tripplet(d):
@@ -45,6 +48,8 @@ def handle_list(options, _, config):
 
     if options.json:
         print(json.dumps(ret.json()))
+    elif options.json_out:
+        options.json_out.write(json.dumps(ret.json(), indent=4))
     else:
         for prj in ret.json()["results"]:
             print(f"* {prj['name']}")
@@ -66,10 +71,22 @@ def setup_parser(parser):
     # "project get <name>"
     p = parser.add_parser("get")
     p.add_argument("name")
-    p.add_argument("--json", action="store_true")
+    p.add_argument("--json", action="store_true", help="Output json project to stdout")
+    p.add_argument(
+        "--json-out",
+        help="Write json project out to a named file path",
+        type=argparse.FileType("w", encoding="utf-8"),
+    )
 
     # "project list"
     p = parser.add_parser("list")
-    p.add_argument("--json", action="store_true")
+    p.add_argument(
+        "--json", action="store_true", help="Output json project list to stdout"
+    )
+    p.add_argument(
+        "--json-out",
+        help="Write json project list out to a named file path",
+        type=argparse.FileType("w", encoding="utf-8"),
+    )
 
     return sorted(parser._name_parser_map.keys())
