@@ -2,6 +2,7 @@
 
 import requests
 
+from tuxsuite.utils import is_down
 from tuxsuite.cli.version import __version__
 
 
@@ -16,6 +17,7 @@ def apiurl(config, url):
     return f"{config.tuxapi_url}{url}"
 
 
+@is_down
 def get(config, url, params=None):
     # print(f"GET {config.tuxapi_url}{url} {params=}")
     return requests.get(
@@ -23,6 +25,13 @@ def get(config, url, params=None):
     )
 
 
+@is_down
+def get_storage(config, url, params=None):
+    # print(f"GET {config.tuxapi_url}{url} {params=}")
+    return requests.get(f"{url}", headers=headers(config), params=params)
+
+
+@is_down
 def post(config, url, data):
     # print(f"POST {config.tuxapi_url}{url}")
     # print(f"POST {data}")
@@ -31,11 +40,15 @@ def post(config, url, data):
     )
 
 
-def delete(config, url):
+@is_down
+def delete(config, url, data):
     # print(f"DELETE {config.tuxapi_url}{url}")
-    return requests.delete(f"{config.tuxapi_url}{url}", headers=headers(config))
+    return requests.delete(
+        f"{config.tuxapi_url}{url}", headers=headers(config), json=data
+    )
 
 
+@is_down
 def put(config, url, data):
     # print(f"PUT {config.tuxapi_url}{url}")
     # print(f"PUT {data}")
