@@ -5,8 +5,8 @@ import argparse
 import tuxsuite
 
 from itertools import chain
+from tuxsuite.utils import error
 from tuxsuite.cli.utils import (
-    error,
     format_result,
     format_plan_result,
 )
