@@ -1,12 +1,14 @@
 # -*- coding: utf-8 -*-
 
 import argparse
+import pathlib
 import tuxsuite
 import tuxsuite.exceptions
 import tuxsuite.cli.colors as colors
 import tuxsuite.cli.icons as icons
+from tuxsuite.utils import error
 from tuxsuite.cli.models import Test
-from tuxsuite.cli.requests import apiurl, get, post
+from tuxsuite.cli.requests import apiurl, get, post, headers, get_storage
 from tuxsuite.cli.utils import (
     LIMIT,
     datediff,
@@ -14,12 +16,15 @@ from tuxsuite.cli.utils import (
     key_value,
     wait_for_object,
     format_result,
-    error,
     is_url,
+    overlay_action,
+    get_cb_headers,
+    validate_notify_emails,
 )
 from tuxsuite.cli.yaml import yaml_load
 
 import json
+import requests
 import sys
 import time
 
@@ -50,12 +55,33 @@ def handle_get(options, _, config):
     test = Test.new(**ret.json())
     if options.json:
         print(test.as_json())
+    elif options.json_out:
+        options.json_out.write(json.dumps(test.as_dict(), indent=4))
+    elif options.list_artifacts:
+        path_url = (
+            f"{test.download_url}{options.list_artifacts}/"
+            if options.list_artifacts is not True
+            else test.download_url
+        )
+        ret = get_storage(config, path_url).json()
+        for folder in ret.get("folders", []):
+            print(f"📁 {folder['Url'].split('/')[-2]}")
+        for file in ret.get("files", []):
+            print(f"📄 {file['Url'].split('/')[-1]}")
+        if options.download_artifacts:
+            tuxsuite.download.download_artifacts(
+                config, test, options.download_artifacts, options.list_artifacts, ret
+            )
+    elif options.download_artifacts:
+        tuxsuite.download.download_artifacts(config, test, options.download_artifacts)
     else:
         print(f"url     : {apiurl(config, test.url())}")
         print(f"project : {test.project}")
         print(f"uid     : {test.uid}")
         print(f"plan    : {test.plan}")
-        print(f"build   : {test.waiting_for}")
+        if test.waiting_for:
+            kind, waiting_for = test.waiting_for.split("#")
+            print(f"{kind.lower()}   : {waiting_for}")
         print(f"user    : {test.user}")
 
         print(f"device  : {test.device}")
@@ -63,6 +89,7 @@ def handle_get(options, _, config):
         print(f"modules : {test.modules}")
         print(f"bootargs: {test.boot_args}")
         print(f"tests   : {', '.join(test.tests)}")
+        print(f"overlays : {test.overlays}")
 
         if test.provisioning_time:
             print(f"{icons.PROVISIONING} time : {test.provisioning_time}")
@@ -116,17 +143,37 @@ def handle_list(options, _, config):
     if ret.status_code != 200:
         raise NotImplementedError()
 
-    tests = [Test.new(**t) for t in ret.json()["results"][: options.limit]]
+    tests = [Test.new(**t) for t in ret.json()["results"]]
     n_token = ret.json()["next"]
-    if options.json:
-        print(json.dumps([t.as_dict() for t in tests]))
+    if options.json or options.json_out:
+        itrs = options.limit // LIMIT
+        for _ in range(itrs):
+            if not n_token:
+                break
+            items, n_token = fetch_next(Test, config, url, n_token)
+            tests.extend(items)
+        if options.json:
+            print(json.dumps([t.as_dict() for t in tests[: options.limit]]))
+        elif options.json_out:
+            options.json_out.write(
+                json.dumps([t.as_dict() for t in tests[: options.limit]], indent=4)
+            )
     else:
+        previous_pt = None
         while True:
-            previous_pt = None
-            for test in tests:
+            while len(tests) < options.limit and n_token:
+                items, n_token = fetch_next(Test, config, url, n_token)
+                tests.extend(items)
+
+            if not tests:
+                break
+
+            for test in tests[: options.limit]:
                 state = test.result if test.state == "finished" else test.state
                 state_msg = (
-                    f"{colors.state(test.state, test.result)}{state}{colors.reset}"
+                    (f"{colors.state(test.state, test.result)}{state}{colors.reset}")
+                    if sys.stdout.isatty()
+                    else state
                 )
                 all_tests = ",".join(test.tests)
                 pt = test.provisioning_time
@@ -138,16 +185,31 @@ def handle_list(options, _, config):
                 )
 
                 previous_pt = pt
+            tests = tests[options.limit :]
             if sys.stdout.isatty():
-                # fetch next list of tests
-                tests, n_token = fetch_next(Test, config, url, n_token, options.limit)
+                try:
+                    input(
+                        "\nPress [ENTER] to see next list of tests, or Ctrl-C to quit:\n"
+                    )
+                except KeyboardInterrupt:
+                    sys.exit(0)
     return 0
 
 
 def handle_logs(options, _, config):
     ret = get(
         config,
-        f"/v1/groups/{config.group}/projects/{config.project}/tests/{options.uid}/logs/lava",
+        f"/v1/groups/{config.group}/projects/{config.project}/tests/{options.uid}",
+    )
+    if ret.status_code == 404:
+        print(f"warning: {colors.red}Log unavailable{colors.reset}")
+        return 0
+    elif ret.status_code != 200:
+        raise NotImplementedError()
+
+    # TODO: add a wrapper around requests.get
+    ret = requests.get(
+        f"{ret.json()['download_url']}lava-logs.yaml", headers=headers(config)
     )
     if ret.status_code != 200:
         raise NotImplementedError()
@@ -171,8 +233,19 @@ def handle_logs(options, _, config):
 def handle_results(options, _, config):
     ret = get(
         config,
-        f"/v1/groups/{config.group}/projects/{config.project}/tests/{options.uid}/results",
+        f"/v1/groups/{config.group}/projects/{config.project}/tests/{options.uid}",
     )
+    if ret.status_code == 404:
+        print(f"warning: {colors.red}Log unavailable{colors.reset}")
+        return 0
+    elif ret.status_code != 200:
+        raise NotImplementedError()
+
+    # TODO: add a wrapper around requests.get
+    ret = requests.get(
+        f"{ret.json()['download_url']}results.json", headers=headers(config)
+    )
+
     if ret.status_code != 200:
         raise NotImplementedError()
 
@@ -181,18 +254,19 @@ def handle_results(options, _, config):
         return 0
 
     data = json.loads(ret.text)
-    for k1, v2 in data.items():
-        for k2, v2 in v2.items():
-            if v2["result"] == "pass":
-                print(f"{k1}.{k2}: {colors.green}{v2['result']}{colors.reset}")
-            elif v2["result"] == "fail":
-                print(f"{k1}.{k2}: {colors.red}{v2['result']}{colors.reset}")
-            elif v2["result"] == "canceled":
-                print(f"{k1}.{k2}: {colors.white}{v2['result']}{colors.reset}")
-            elif v2["result"] == "unknown":
-                print(f"{k1}.{k2}: {colors.purple}{v2['result']}{colors.reset}")
-            else:
-                print(f"{k1}.{k2}: {v2['result']}")
+    for k1, v1 in data.items():
+        for k2, v2 in v1.items():
+            for k3, v3 in v2.items():
+                if v3 == "pass":
+                    print(f"{k1}.{k2}: {colors.green}{v3}{colors.reset}")
+                elif v3 == "fail":
+                    print(f"{k1}.{k2}: {colors.red}{v3}{colors.reset}")
+                elif v3 == "canceled":
+                    print(f"{k1}.{k2}: {colors.white}{v3}{colors.reset}")
+                elif v3 == "skip":
+                    print(f"{k1}.{k2}: {colors.yellow}{v3}{colors.reset}")
+                elif v3 == "unknown":
+                    print(f"{k1}.{k2}: {colors.purple}{v3}{colors.reset}")
     return 0
 
 
@@ -213,7 +287,9 @@ def handle_wait(options, _, config):
             print(f"project : {test.project}")
             print(f"uid     : {test.uid}")
             print(f"plan    : {test.plan}")
-            print(f"build   : {test.waiting_for}")
+            if test.waiting_for:
+                kind, waiting_for = test.waiting_for.split("#")
+                print(f"{kind.lower()}   : {waiting_for}")
             print(f"user    : {test.user}")
 
             print(f"device  : {test.device}")
@@ -221,6 +297,7 @@ def handle_wait(options, _, config):
             print(f"modules : {test.modules}")
             print(f"bootargs: {test.boot_args}")
             print(f"tests   : {', '.join(test.tests)}")
+            print(f"overlays : {test.overlays}")
             if test.provisioning_time:
                 print(f"{icons.PROVISIONING} time : {test.provisioning_time}")
             if test.running_time:
@@ -295,6 +372,15 @@ def handle_submit(cmdargs, _, config):
     for t in cmdargs.timeouts:
         timeouts_d[t[0][0]] = int(t[0][1])
 
+    cb_headers = None
+    if cmdargs.callback_header:
+        if not cmdargs.callback:
+            error("--callback-header given without a --callback")
+        else:
+            cb_headers = get_cb_headers(cmdargs.callback_header)
+
+    validate_notify_emails(cmdargs.notify_email)
+
     try:
         test = tuxsuite.Test(
             ap_romfw=cmdargs.ap_romfw,
@@ -303,21 +389,31 @@ def handle_submit(cmdargs, _, config):
             device=cmdargs.device,
             dtb=cmdargs.dtb,
             fip=cmdargs.fip,
+            job_definition=cmdargs.job_definition,
             kernel=cmdargs.kernel,
             mcp_fw=cmdargs.mcp_fw,
             mcp_romfw=cmdargs.mcp_romfw,
             modules=cmdargs.modules,
+            overlays=cmdargs.overlays,
             parameters=params,
             rootfs=cmdargs.rootfs,
             scp_fw=cmdargs.scp_fw,
             scp_romfw=cmdargs.scp_romfw,
             tests=tests,
+            test_name=cmdargs.test_name,
             timeouts=timeouts_d,
             wait_for=cmdargs.wait_for,
             callback=cmdargs.callback,
+            callback_headers=cb_headers,
             commands=cmdargs.commands,
             qemu_image=cmdargs.qemu_image,
+            shared=cmdargs.shared,
+            host=cmdargs.host,
             tuxbuild=cmdargs.tuxbuild,
+            lab=cmdargs.lab,
+            lava_test_plans_project=cmdargs.lava_test_plans_project,
+            is_public=cmdargs.private,
+            notify_emails=cmdargs.notify_email,
         )
     except (AssertionError, tuxsuite.exceptions.TuxSuiteError) as e:
         error(e)
@@ -333,14 +429,11 @@ def handle_submit(cmdargs, _, config):
     if cmdargs.no_wait:
         format_result(test.status, test.url)
     else:
-        try:
-            test_result = wait_for_object(test)
-        except tuxsuite.exceptions.Timeout as exc:
-            test_result = False
-            sys.stderr.write(f"Error: {str(exc)}\n")
+        test_result = wait_for_object(test)
 
-    if cmdargs.json_out:
-        cmdargs.json_out.write(json.dumps(test.status, sort_keys=True, indent=4))
+    if cmdargs.json_out and test.status:
+        with open(cmdargs.json_out, "w") as json_out:
+            json_out.write(json.dumps(test.status, sort_keys=True, indent=4))
 
     # If the test did not pass, exit with exit code of 1
     if not test_result:
@@ -419,6 +512,25 @@ def test_cmd_options(sp):
         type=str,
     )
     sp.add_argument("--fip", help="URL of the fip.bin to test", default=None, type=str)
+    sp.add_argument(
+        "--job-definition",
+        help="URL/File of the lava job definition to test. Applicable to lava-fvp device type only",
+        default=None,
+        type=str,
+    )
+    sp.add_argument(
+        "--overlay",
+        default=[],
+        metavar="URL/String",
+        type=str,
+        help=(
+            "Tarball with overlay and optionally PATH to extract the tarball,"
+            " default PATH '/'. Overlay can be specified multiple times"
+        ),
+        action=overlay_action,
+        nargs="+",
+        dest="overlays",
+    )
     sp.add_argument(
         "--parameters",
         help="test parameters as KEY=VALUE",
@@ -463,7 +575,14 @@ def test_cmd_options(sp):
     sp.add_argument(
         "--json-out",
         help="Write json test status out to a named file path",
-        type=argparse.FileType("w", encoding="utf-8"),
+        type=pathlib.Path,
+    )
+    sp.add_argument(
+        "-P",
+        "--private",
+        default=True,
+        action="store_false",
+        help="Private Test",
     )
     sp.add_argument(
         "--callback",
@@ -474,6 +593,18 @@ def test_cmd_options(sp):
         ),
         type=is_url,
     )
+    sp.add_argument(
+        "--callback-header",
+        help=(
+            "Extra header to include in the POST request sent by the test "
+            "backend to the callback URL. The header string should be a key "
+            "value pair separated by a ':' (colon). This option can be used "
+            "multiple times to add multiple headers. "
+            "Example: --callback-header 'X-First-Name: Joe'"
+        ),
+        type=str,
+        action="append",
+    )
     sp.add_argument(
         "--commands",
         nargs="*",
@@ -485,6 +616,26 @@ def test_cmd_options(sp):
         default=None,
         help="Use qemu from the given container image",
     )
+    sp.add_argument(
+        "--shared",
+        action="store_true",
+        help="Publish artefacts stored by testcase in /mnt/tuxrun directory. Applies to qemu devices only.",
+    )
+    sp.add_argument(
+        "--host",
+        metavar="HOST",
+        default="x86_64",
+        help=(
+            "Allow user to choose the host Runner. "
+            "Default value is 'x86_64'. "
+            "Valid values 'x86_64', 'm6a.large', 'm6id.large', 'm7g.large', 'm7gd.large'. "
+            "x86_64 is 2vCPU + 4GB RAM with swap. "
+            "m6a.large is 2vCPU + 8GB RAM with no swap. "
+            "m7g.large is 2vCPU + 8GB RAM with no swap. "
+            "m6id.large is 2vCPU + 8GB RAM with NVMe disk and no swap. "
+            "m7gd.large is 2vCPU + 8GB RAM with NVMe disk and no swap. "
+        ),
+    )
     sp.add_argument(
         "--tuxbuild",
         metavar="URL",
@@ -492,17 +643,77 @@ def test_cmd_options(sp):
         help="URL of a TuxBuild build",
         type=is_url,
     )
+    sp.add_argument(
+        "--lab",
+        metavar="URL",
+        default="https://lkft.validation.linaro.org",
+        help="URL of LAVA lab instance",
+        type=is_url,
+    )
+    sp.add_argument(
+        "--lava-test-plans-project",
+        default=None,
+        help="Lava test plans project name",
+        type=str,
+    )
+    sp.add_argument(
+        "--test-name",
+        help="User defined string to identify the test",
+        type=str,
+    )
+    sp.add_argument(
+        "-E",
+        "--notify-email",
+        help=(
+            "Email address to be notified once the respective test completes. "
+            "This option can be used multiple times to add multiple "
+            "notification email addresses."
+        ),
+        type=str,
+        action="append",
+    )
 
 
 def setup_parser(parser):
     # "test get <uid>"
     t = parser.add_parser("get")
     t.add_argument("uid")
-    t.add_argument("--json", action="store_true")
+    grp = t.add_mutually_exclusive_group()
+    grp.add_argument("--json", action="store_true", help="Output json test to stdout")
+    grp.add_argument(
+        "--json-out",
+        help="Write json test out to a named file path",
+        type=argparse.FileType("w", encoding="utf-8"),
+    )
+    t.add_argument(
+        "-l",
+        "--list-artifacts",
+        nargs="?",
+        default=False,
+        const=True,
+        help="List the test artifacts(files and directories) with maxdepth=1. Path (optional) can be passed "
+        "to list the test artifacts from specified path",
+    )
+    t.add_argument(
+        "-d",
+        "--download-artifacts",
+        type=pathlib.Path,
+        nargs="?",
+        const=".",
+        help="Download the test artifacts(files only) with maxdepth=1. Path (optional) can be passed "
+        "to download the test artifacts at specified location. (default: current directory)",
+    )
 
     # "test list"
     t = parser.add_parser("list")
-    t.add_argument("--json", action="store_true")
+    t.add_argument(
+        "--json", action="store_true", help="Output json test list to stdout"
+    )
+    t.add_argument(
+        "--json-out",
+        help="Write json test list out to a named file path",
+        type=argparse.FileType("w", encoding="utf-8"),
+    )
     t.add_argument("--limit", type=int, default=LIMIT)
 
     # "test logs <uid>"
