@@ -58,7 +58,7 @@ class Config:
 
         Config file:
             Must be located at ~/.config/tuxsuite/config.ini.
-            This location can be overriden by setting the TUXSUITE_CONFIG
+            This location can be overridden by setting the TUXSUITE_CONFIG
             environment variable.
             A minimum config file looks like:
 
@@ -76,6 +76,8 @@ class Config:
                 api_url (optional)
                 tuxapi_url (optional)
                 tuxauth_url (optional)
+                lava_test_plans_project (optional)
+                lab (optional)
         """
 
         # defaults
@@ -83,8 +85,10 @@ class Config:
         self.group = None
         self.project = None
         self.kbapi_url = "https://api.tuxbuild.com/v1"
-        self.tuxapi_url = "https://tuxapi.tuxsuite.com"
-        self.tuxauth_url = "https://auth.tuxsuite.com"
+        self.lava_test_plans_project = None
+        self.lab = None
+        self.tuxapi_url = os.getenv("TUXAPI_URL", "https://tuxapi.tuxsuite.com")
+        self.tuxauth_url = os.getenv("TUXAUTH_URL", "https://auth.tuxsuite.com")
 
         # tuxsuite environment
         self.tuxsuite_env = getenv("ENV", "default")
@@ -104,12 +108,18 @@ class Config:
             self.kbapi_url = config.get("api_url", self.kbapi_url).rstrip("/")
             self.tuxapi_url = config.get("tuxapi_url", self.tuxapi_url).rstrip("/")
             self.tuxauth_url = config.get("tuxauth_url", self.tuxauth_url).rstrip("/")
+            self.lava_test_plans_project = config.get("lava_test_plans_project")
+            self.lab = config.get("lab")
 
         # override with configuration from env variables
         self.auth_token = getenv("TOKEN", self.auth_token)
         self.group = getenv("GROUP", self.group)
         self.project = getenv("PROJECT", self.project)
         self.kbapi_url = getenv("URL", self.kbapi_url).rstrip("/")
+        self.lava_test_plans_project = os.getenv(
+            "LAVA_TEST_PLANS_PROJECT", self.lava_test_plans_project
+        )
+        self.lab = os.getenv("LAB", self.lab)
 
         # token and kbapi should be specified
         if not self.auth_token:
@@ -221,6 +231,7 @@ class PlanConfig:
         self.plan = []
         self.schema_warning = ""
         self.plan_type = None
+        self.plan_file = None
         self.__load_config__()
 
     def __load_config__(self):
@@ -233,6 +244,9 @@ class PlanConfig:
         except (FileNotFoundError, yaml.loader.ParserError) as e:
             raise InvalidConfiguration(str(e))
 
+        # Raw plan file content
+        self.plan_file = contents
+
         # calling Parent Plan Type class to get respective plan type obj ( kernel or bake plan)
         self.plan_type = tuxsuite.PlanType.load_plan(config)
 
