@@ -1,6 +1,15 @@
 # -*- coding: utf-8 -*-
 
+import sys
+
+
 from attr import attrs, attrib
+from requests.exceptions import HTTPError
+
+
+def error(msg):
+    sys.stderr.write(f"Error: {msg}\n")
+    sys.exit(1)
 
 
 class defaults:
@@ -58,3 +67,16 @@ result_states = {
         cli_color="cyan",
     ),
 }
+
+
+def is_down(func):
+    def wrapper(*args, **kwargs):
+        try:
+            ret = func(*args, **kwargs)
+            ret.raise_for_status()
+        except HTTPError as exc:
+            if exc.response.status_code == 503:
+                error(exc)
+        return ret
+
+    return wrapper
